#ifndef _EquipDcl_h_
#define _EquipDcl_h_ 1

/** EquipDcl.h creation : 24 octobre 97 last modification : 29 april 99
 *
 * Definition du format de fichier Data de description des equipements sur un 
 * systeme d'acquisition GANIL autre que PDOS
 *
 *    Modifications :
 *         22 juin 98 - B. Raine
 *           Modif ordre de declaration CAMAC
 *         29 april 99 - B. Raine
 *           D2VBDEF
 *
 *
*/
/*
*
* Macro CPUDEF definit un descripteur de CPU dans le cadre de l'acquisition 
* multidetecteur GANIL.
* Appel :
*  CPUDEF cpuname,num,ip_nam,cputyp,appltyp,SHVMEadr,SHCPUadr,SHmemsiz
* avec :cpunam  = nom du cpu dans le chassis
*       num     = numero de l'equipement dans la famille
*       ip_name = nom internet du cpu (si absent, pas d'Ethernet)
*	cputyp  = type de CPU (ex : F21 pour cpu21 de FORCE) (si absent erreur)
*	appltyp = type d'application (ACQ, TRA, DEV...) (si absent NO_APPL)
*	SHVMEadr= adresse de la memoire partagee avec l'exterieur
*		  vue du VME (0 si absente)
*	SHCPUadr= adresse de la memoire partagee avec l'exterieur
*		  vue du processeur 
*                 0 si absente
*                 Obligatoire si SHVMEadr
*	SHmemsiz= taille de la memoire  partagee en bytes 
*                 0 si absente
*                 Obligatoire si SHVMEadr
*

*
* Macro VMEMEMDEF decrit une zone memoire VME.
* Appel :
*  VMEMEMDEF name,adrvme,siz,am 
*	name	= nom de la carte
*	adrvme	= adresse vme de la memoire
*	siz	= taille de la zone memoire
*       am      = am code
*
*

*
* Macro MEMDEF decrit une zone memoire VME.
* Appel :
*  MEMDEF name,BoardName,siz
*	name	= nom de la zone memoire
*       BoardName = Nom de la carte 
*	siz	= taille de la zone memoire
*
*

*
* Macro CAMDEF decrit un coupleur de branche CAMAC
* Appel :
*  CAMDEF name,NumBranchMin,NbBranch,adr,type,cpu
*	name	= nom de la carte
*       NumBranchMin = Numero de branche min traite par ce coupleur
*       NbBranch     = Nombre de branches traitees par ce coupleur
*	adr	= adresse de la carte
*	type    = type de coupleur CAMAC VME ou VSB, soit: CBD_8210 ou CBD_8110
*	cpu	= nom du cpu associe a ce coupleur
*


*
* Macro VICDEF decrit une carte VIC.
* Appel :
*  VICDEF name,num,type,[VsbCpuName]
*	name	= nom de la carte VIC
*       num     = numero de l'equipement dans la famille
*       type    = type de la carte : VIC_8250, VIC_8251
*       VsbCpuName = Nom de la carte CPU connectee au bus VSB
*


*
* Macro VXIDEF decrit un ensemble de chassis VXI.
* Appel :
*  VXIDEF name, FirstCrate, NbCrate, AccessInterface, VmeBaseAdr, Size, Am
*	name	= nom de l'ensemble VXI
*       FirstCrate = numero 1er chassis
*       NbCrate    = nombre de chassis
*       AccessInterface = Nom de l'interface d'acces au VXI 
*                         par exemple, nom d'une carte VIC ou argument vide si 
*                         le processeur de trouve dans le chassis VXI
*       VmeBaseAdr = Adresse VME de base allouee a cet ensemble VXI
*       Size       = Taille en octets de la zone VME utilisable 
*       Am         = Am code pour acceder au VXI
*
*

*
* Macro IFVGLOB decrit les parametres communs a l'ensemble des cartes IFV du
* syteme
* Appel :
*  IFVGLOB name,baseadr,bufsiz,bufnb
*	name	= nom sur 4 caracteres de l'ensemble de cartes IFV considere
*	baseadr = adresse de base
*	bufsiz	= taille des buffers
*	bufnb	= nombre de buffers
*

*
* Macro IFVDEF definit le nom et le numero d'une carte IFV
* Appel :
*  IFVDEF name,globname,num
*	name	= nom de la carte
*       globnam = nom de l'ensemble IFV considere, decrit par une cde IFVGLOB
*	num	= numero
*

*
* Macro D2VBDEF definit une carte D2VB
* Appel :
*  D2VBDEF name, RegBaseAddr, RamBaseAddr, BufSize
*       name        = nom de la carte
*       RegBaseAddr = Register base address
*       RamBaseAddr = Buffer RAM base address
*       BufSize     = Buffer size in bytes
*

*
* Macro PROCDEF decrit un Process d'acquisition
*   PROCDEF,Name,CpuName,[TimeoutSend],[TimeoutReceive],
	    ProcType,ExeName,[DefBufSize],
            InterfaceList or InputBufstack,
            IPaddress
*       Name           = process name
*       CpuName        = cpu running the process
*       TimeoutSend    = Timeout in seconds for message passing
*       TimeoutReceive = Timeout in seconds for message passing
*       ProcType       = process type : DataInput
*                                       Merge
*                                       DataProc
*                                       DataOutput
*       ExeName        = name of the executable file
*       DefBufSize     = default output buffer size
*       InterfaceList  = list of data input interfaces if ProcType = DataInput
*       InputBufstack  = name of the input bufstack if ProcType != DataInput
*       IPaddress      = IP address if ProcType = DataOutput and 
*                        we want use no standard ethernet network interface
*

*
* Macro MEXTDEF decrit une zone memoire VME destinee a l'acquisition d'un
* coupleur extren vu a travers une memoire partagee.
* Appel :
*  MEXTDEF name,num,adr,siz,OptFifo,AmbxName,OptSynchro
*	name	= nom de la carte
*	num	= numero
*	adr	= adresse de la memoire
*	siz	= taille de la zone memoire
*	OptFifo = Organisation de la memoire evts, valeurs possibles :
*			- MextNoFifo  : pas de FIFO
*			- MextGanFifo : Fifo de type GANIL Acquisition
*	AmbxName= Nom mailbox acquisition associee ("" si pas de mailbox)
*	OptSyncro = Options de synchro par mailbox, valeurs possibles :
*			- NoSync    : pas de synchro
*                       - AllSync   : toutes les options de synchro
*			- InstSync  : synchro a l'installation de l'acq
*			- StartSync : synchro au start de l'acquisition
*       		- StopSync  : synchro au stop de l'acquisition
*

*
* Macro AMBXDEF decrit une zone boite aux lettres acquisition pour synchro
* avec coupleur externe.
* Appel :
*  AMBXDEF name,num,adr,ItAdr,ItSiz,ItVal,Timeout
*	name	= nom de la mailbox
*	num	= numero
*	adr	= adresse de la mailbox (peut-etre egale a 0 si non definie)
*	ItAdr	= adresse du mot d'interrupt si il existe (0 sinon)
*	ItSiz   = taille du mot d'interrupt
*	ItVal	= valuer a ecrire dans mot d'interrupt
*	Timeout = timeout en ms pour recevoir AMBX_ACK_EXEC (commande recue)
*

*
* Macro CNETDEF decrit un contoleur VME CAENET.
* Appel :
*  CNETDEF name,num,adr,type,ident,mycpu
*	name	= nom de la carte
*	num	= numero
*	adr	= adresse de base du module
* 	type	= type du controleur :
*		  - V288 = Highspeed  
*		  - A200 = Lowspeed 
*	ident   = ident du controleur
*       mycpu   = nom du cpu gerant associe au controleur CAENET


*
* Macro HTDEF decrit un chassis haute tension
* Appel :
*  HTDEF name,num,nocrate,nbmod,type,mycntrl
*	name	= nom de l'equipement
*	num	= numero de l'equipement
*       nocrate = numero du chassis sur le CAENET
*	nbmod	= nbre de modules installes dans le chassis	
* 	type	= type du chassis HT :
*		  - SY127 = chassis de 10 modules de  4 voies
*		  - SY403 = chassis de  4 modules de 16 voies
*       mycntrl   = nom du controleur CAENET gerant le chassis


*
* Macro ECHDEF decrit un module VME echelle
* Appel :
*  ECHDEF name,num,adr,type,mycpu
*	name	= nom de l'equipement
*	num	= numero
*	adr	= adresse de base du module
* 	type	= type de l'echelle
*		  - V260 = 16 voies 24 bits
*		  - Lecroy
*=====> A REVOIR!!!==>       mycpu   = nom du cpu associe aux echelles


*/
#define EquipLineLen 256    /* taille max d'une ligne de cde dans un 
                               fichier d'equipements */
#define EquipComment '*'    /* Caractere delimitant une ligne de commentaires */

/* Definition des verbes */
#define EQUIPSIZE_VERBE "EQUIPSIZE"
#define BADDEF     ""
#define CPUDEF     "CPUDEF"
#define VMEMEMDEF  "VMEMEMDEF"
#define MEMDEF     "MEMDEF"
#define IFVGLOB    "IFVGLOB"
#define IFVDEF     "IFVDEF"
#define MEXTDEF    "MEXTDEF"
#define AMBXDEF    "AMBXDEF"
#define CAMDEF     "CAMDEF"
#define CNETDEF    "CNETDEF"
#define HTDEF      "HTDEF"
#define ECHDEF     "ECHDEF"
#define VICDEF     "VICDEF"
#define VXIDEF     "VXIDEF"
#define D2VBDEF    "D2VBDEF"
#define PROCDEF    "PROCDEF"

/* Prototypes des fonctions de traitement des verbes */

#ifndef __EquipDcl_c
extern
#endif
int EquipDclFile (FILE *fd, EQUIPHD *EquipHd, int Verbose);
#ifndef __EquipDcl_c
extern
#endif
int EquipDclLine (char *Line, EQUIPHD *EquipHd, int Verbose);

#ifndef __EquipDcl_c
extern
#endif
void EquipDclError  (char * ErrMes);
#ifndef __EquipDcl_c
extern
#endif
void EquipDclWarning(char * ErrMes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclNoImpl  (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes); /* cde non implementee */

#ifndef __EquipDcl_c
extern
#endif
int EquipDclBad     (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes); /* Bad command */

#ifndef __EquipDcl_c
extern
#endif
int EquipDclNum     (char ** pLine, EQUIPDES *EquipDes); /* Traite l'argument num */

#ifndef __EquipCpu_c
extern
#endif
int EquipDclCpu     (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclIfvGlob (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclIfv     (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipVmeMem_c
extern
#endif
int EquipDclVmeMem  (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipMem_c
extern
#endif
int EquipDclMem     (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclMext    (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclAmbx    (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclCam     (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclCnet    (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclHt      (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDcl_c
extern
#endif
int EquipDclEch     (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipVic_c
extern
#endif
int EquipDclVic     (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipVxi_c
extern
#endif
int EquipDclVxi     (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDclD2vb_c
extern
#endif
int EquipDclD2vb    (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipProc_c
extern
#endif
int EquipDclProc    (char * Line, EQUIPHD *EquipHd, EQUIPDES *EquipDes);

#ifndef __EquipDclVerif_c
extern
#endif
int EquipDclVerifBranch(int FirstBranch, int NbBranch);

#ifndef __EquipDclVerif_c
extern
#endif
int EquipDclVerifVicCrate(int FirstCrate, int NbCrate);

#ifndef __EquipCheck_c
extern
#endif
int EquipCheck    (void);

#endif













