
  /* Definition of Acquisition processes types */

*
* Macro PROCDEF decrit un Process d'acquisition
*   PROCDEF,Name,CpuName,[TimeoutSend],[TimeoutReceive],
	    ProcType,ExeName,[DefBufSize],
            InterfaceList or InputBufstack,
            IPaddress
*       Name           = process name
*       CpuName        = cpu running the process
*       TimeoutSend    = Timeout in seconds for message passing
*       TimeoutReceive = Timeout in seconds for message passing
*       ProcType       = process type : DataReadout
*                                       Merge
*                                       DataProc
*                                       DataTrans
*       ExeName        = name of the executable file
*       DefBufSize     = default output buffer size
*       InterfaceList  = list of data readout interfaces if ProcType = Readout
*       InputBufstack  = name of the input bufstack if ProcType != Readout
*       IPaddress      = IP address if ProcType = DataTrans and 
*                        we want use no standard ethernet network interface
*


#define DataReadout        1  /* Data Readout */ 
#define SubEventMerge      2  /* Sub-event merge */
#define DataProcess        4  /* Data Processing */
#define DataTrans          8  /* Data Transfer to the host */

  /* Definition of the maximum numbers of interfaces per readout process */
#define INTERFACE_MAX_NB 4

  typedef struct READOUTDES {     /* Readout process descriptor */
    UNSINT32 InterfaceNb;      /* Number of interfaces */
    char InterfaceName[INTERFACE_MAX_NB][EQUIP_NAME_LEN];
  } READOUTDES;

  typedef struct MERGEDES {     /* Merge process descriptor */
    char InputBufstack[EQUIP_NAME_LEN]; /* Input bufstack name */
    int  BufstackFifo;          /* Fifo number for input bufstack */
    int  BufstackNb;            /* Number of input bufstacks */
  } MERGEDES;

  typedef struct DATAPROCDES {  /* Data processing process descriptor */
    char InputBufstack[EQUIP_NAME_LEN]; /* Input bufstack name */
    int  BufstackFifo;          /* Fifo number for input bufstack */
    int  BufstackNb;            /* Number of input bufstacks */
  } DATAPROCDES;

  typedef struct TRANSDES {     /* Data transfer process descriptor */
    char InputBufstack[EQUIP_NAME_LEN]; /* Input bufstack name */
    int  BufstackFifo;          /* Fifo number for input bufstack */
    int  BufstackNb;            /* Number of input bufstacks */
    UNSINT32 protocol;             /* Protocole de dialogue */
    char     IPname[EQUIP_NAME_LEN];   /* Nom Internet */
  } TRANSDES;

  typedef struct PROCDES {     /* Acquisition process descriptor */
    char CpuName[EQUIP_NAME_LEN];  /* Cpu name executing the process */
    UNSINT32 CpuId;
    UNSINT32 ToSend;               /* Timeout send for message passing */
    UNSINT32 ToReceive;            /* Timeout receive for message passing */
    UNSINT32 DefOutputBufSize;     /* Default output buffer size */
    char ExeName[EXE_NAME_LEN];    /* Executable name */
    UNSINT32 ProcType;             /* Process type */
    union {
      READOUTDES  Readout;
      MERGEDES    Merge;
      DATAPROCDES DataProc;
      TRANSDES    Trans;
    } type;
  } PROCDES;
