/**********************************************************/
/*                                                        */
/*                Package Hermes                          */
/*                                                        */
/* ErrSend.c                                              */
/*                                                        */
/* Copyright GANIL, Groupe Informatique Physique, 1999    */
/*                                                        */
/* Auteur : Frederic Saillant                             */
/*                                                        */
/* Version : v1r1 (EXOGAM)                                */
/*                                                        */
/* Date de creation : 14 mai 1999                         */
/*                                                        */
/* Derniere mise a jour : 1er juin 1999                   */
/*                                                        */
/* Fonction de signalement d'une erreur (ou d'un message).*/
/*                                                        */
/* Synopsis :  int ErrSend ( int    ErrCode,              */
/*                           int    ErrType,              */
/*                           char * Source,               */
/*                           char * ExtraInfo )           */
/**********************************************************/

#define _POSIX_SOURCE /* on veut des sources POSIX */
#define _POSIX_C_SOURCE 199506L

typedef unsigned char  u_char;  /* cette definition dans types.h n'est pas POSIX ! */
typedef unsigned short u_short; /* cette definition dans types.h n'est pas POSIX ! */
typedef unsigned int   u_int;   /* cette definition dans types.h n'est pas POSIX ! */

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#if defined(__sparc) || defined(__osf__)
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/fcntl.h>
#else
#include <socket.h>
#ifdef __Lynx__
#include <sys/file.h>
#else
#include <file.h>
#endif
#endif

#include <errno.h>
#include <time.h>

#ifndef __VMS
extern int gethostname(char *,int); /* seul VMS la declare en POSIX */
#endif

#define _ErrSend_

#include "HermesPrivate.h" /* fonctions privees */
#include "Hermes.h"
#include "HermesDef.h" /* pthread.h est inclus dans HermesDef.h */

int SendToLogger = HERMES_NO_SEND; /* flag "on envoie au logger" */

#ifdef pthread_once_init
static pthread_once_t once = pthread_once_init;
#else
#ifdef PTHREAD_ONCE_INIT
static pthread_once_t once = PTHREAD_ONCE_INIT;
#else
#error "pthread_once_init ou PTHREAD_ONCE_INIT non definis"
#endif
#endif

static void ErrDate ( char * buf ); /* met la date et l'heure dans un buffer */

static int ErrDispatch ( T_ErrMsg * pErrMsg ); /* dispatch le message */

static void ErrDate ( char * buf)
{
  static char *Month[] = {"Jan","Feb","Mar","Apr","May","Jun",
                           "Jul","Aug","Sep","Oct","Nov","Dec"};
  time_t seconds;
  struct tm * p_result;

#if defined(_POSIX_VERSION) && !defined(__VMS)
  struct tm result;

  p_result = &result;
#endif

  time(&seconds);

#ifdef __Lynx__
  localtime_r(p_result,&seconds); /* version LynxOS POSIX septembre 1993 */
#else
#if !defined(__VMS) && ( _POSIX_VERSION >= 199506 )
  localtime_r(&seconds,p_result); /* version POSIX juin 1995 */
#else
  p_result = localtime(&seconds);
#endif
#endif

  sprintf(buf,"%02.2d-%s-%02.2d %02.2d:%02.2d:%02.2d",
          p_result->tm_mday, Month[p_result->tm_mon], p_result->tm_year,
          p_result->tm_hour, p_result->tm_min, p_result->tm_sec );
}

int HermesInit ( char * AppName, int Send )
{
  int ReturnCode = 0;

  T_ClassInfo * pCfg; /* pointeur sur structure d'info de classe */

  pthread_once( &once, ErrMutexCreate ); /* creation de mutex une seule fois */

  ErrMutexLock(); /* lock la mutex du package Hermes */

  SendToLogger = Send;

  pCfg = ErrCfgGet();

  if ( pCfg == NULL ) /* si la config n'est pas initialisee */
  {
    SetHermesApplicationName(AppName);

    ReturnCode = ErrCfgInit();
  }

  ErrMutexUnlock(); /* unlock la mutex */

  return(ReturnCode);
}

int ErrSend ( int ErrCode, int ErrType, char * Source, char * ExtraInfo )
{
  int ReturnCode = 0;
  T_ErrMsg * pErrMsg; /* pointeur sur message d'erreur */
  T_ClassInfo * pCfg; /* pointeur sur structure d'info de classe */
  char errfilepath[MAXFILEPATHLEN]; /* nom complet du fichier des erreurs */
  int len = 0;
  FILE * f;
  char line[512]; /* buffer pour ligne du fichier */
  char * s1;
  char * s2;
  char SysErrMsg[MAXFIELDLEN];
  char SourceSave[MAXFIELDLEN];
  char ExtraSave[MAXFIELDLEN];
  int AppCode; /* Code de l'application */
  char AppName[MAXNAMELEN]; /* Nom de l'application */

  memset((void *)SourceSave,0,MAXFIELDLEN);
  memset((void *)ExtraSave,0,MAXFIELDLEN);

  if (Source)
  {
    strncpy(SourceSave,Source,MAXFIELDLEN-1);
    SourceSave[MAXFIELDLEN - 1] = 0;
  }

  if (ExtraInfo)
  {
    strncpy(ExtraSave,ExtraInfo,MAXFIELDLEN-1);
    ExtraSave[MAXFIELDLEN - 1] = 0;
  }

  strncpy(SysErrMsg,strerror(errno),MAXFIELDLEN-1); /* recupere message erreur systeme */

  SysErrMsg[MAXFIELDLEN - 1] = 0;

  if ( SysErrMsg[( strlen(SysErrMsg) - 1 )] == '\n' )
  {
    SysErrMsg[( strlen(SysErrMsg) - 1 )] = 0;
  }

  pthread_once( &once, ErrMutexCreate ); /* creation de mutex une seule fois */

  ErrMutexLock(); /* lock la mutex du package Hermes */

  pCfg = ErrCfgGet();

  if ( pCfg == NULL ) /* si la config n'est pas initialisee */
  {
    SetHermesApplicationName("");
    ErrCfgInit();
  }

  pCfg = ErrCfgGet();

  AppCode = GetHermesApplicationCode(); /* recuperation du code de l'application */

  strncpy(AppName, GetHermesApplicationName() , MAXNAMELEN - 1 ); /* recuperation du nom de l'application */

  AppName[ MAXNAMELEN - 1 ] = 0;

  ErrMutexUnlock(); /* unlock la mutex */

  if ( pCfg == NULL ) /* si la config n'est toujours pas initialisee */
  {
    ReturnCode = -1;
    printf("Hermes - ErrSend() : Probleme a l'init de la config du package Hermes\n");
  }
  else /* on ne continue que si la config est bien initialisee */
  {
    pErrMsg = (T_ErrMsg *)malloc( sizeof(T_ErrMsg) );

    if ( pErrMsg == NULL )
    {
      ReturnCode = -1;
      printf("Hermes - ErrSend() : Memoire insuffisante\n");
    }
    else
    {
      memset((void *)pErrMsg,0,sizeof(T_ErrMsg));

      pErrMsg->ApplicationCode = AppCode;

      strcpy(pErrMsg->ApplicationName,AppName);

      ErrDate(&pErrMsg->Date[0]);

      pErrMsg->ErrCode = ErrCode;

      pErrMsg->ErrType = ErrType;

      if (Source) strncpy(pErrMsg->Source,SourceSave,MAXFIELDLEN-1);

      pErrMsg->Source[MAXFIELDLEN-1] = 0;

      strncpy(pErrMsg->CallingPackage,pErrMsg->Source,MAXNAMELEN-1);

      pErrMsg->CallingPackage[MAXNAMELEN-1] = 0;

      s1 = pErrMsg->CallingPackage;

      while (
              ( *s1 != 0 ) &&
              (
                ( ( *s1 >= 'A' ) && ( *s1 <= 'Z' ) ) ||
                ( ( *s1 >= 'a' ) && ( *s1 <= 'z' ) ) ||
                ( ( *s1 >= '0' ) && ( *s1 <= '9' ) ) ||
                ( *s1 == '_' ) || ( *s1 == '.' )
	      )
            )
      {
        if ( ( *s1 >= 'a' ) && ( *s1 <= 'z' ) )
	{ /* on passe en majuscules */
          *s1 = *s1 - ( 'a' - 'A' );
        }

        s1++;
      }

      s2 = s1;

      while ( *s2 == ' ' ) /* on saute les espaces eventuels */
      {
        s2++;
      }

      if ( *s2 != '-' )
      {
        pErrMsg->CallingPackage[0] = 0;
      }
      else
      {
        *s1 = 0;
      }

      if (ExtraInfo)
      {
        strncpy(pErrMsg->ExtraInfo,ExtraSave,MAXFIELDLEN);
      }

      pErrMsg->ExtraInfo[MAXFIELDLEN-1] = 0;

      pErrMsg->ProcessId = getpid();

#if ( _POSIX_VERSION >= 199309L )
      pErrMsg->ThreadId = pthread_self();
#else
      pErrMsg->ThreadId = (pthread_t)0;
#endif

      if ( gethostname(pErrMsg->CpuName,MAXNAMELEN) == 0 )
      {
        pErrMsg->CpuName[MAXNAMELEN-1] = 0;
      }
      else
      {
        ReturnCode = -1;
        printf("Hermes - ErrSend() : gethostname() fails !\n");
      }

      if ( ErrCode == -1 )
      {
        memcpy( pErrMsg->Body, SysErrMsg , MAXFIELDLEN );
      }
      else
      {
        pErrMsg->Body[0] = 0;

        pErrMsg->ErrClass = ErrCode >> 16;

        while ( pCfg != NULL )
        {
          if ( pCfg->ClassCode == pErrMsg->ErrClass )
	  {
            strcpy(pErrMsg->ClassPackage, pCfg->PkgName);
            strcpy(errfilepath, pCfg->FilePath);
          }

          pCfg = pCfg->Next;
        }

        if ( pErrMsg->ClassPackage[0] != 0 )
	{
          f = (FILE *)fopen( errfilepath, "r" );

          if ( f != NULL )
	  {
            if ( fgets(line,512,f) != NULL )
	    {
              char ident[20];

              sprintf(ident,"!$%x",pErrMsg->ErrClass);

              if ( strncmp(ident,line,strlen(ident)) == 0 )
	      {
                int found = 0;
                int num;
                char * s;

                while ( (fgets(line,512,f) != NULL) && (found == 0) )
	        {
                  if ( line[0] == '.' )
		  {
                    num = (int)strtol(line+1,&s,16);

                    if ( num == ErrCode )
		    {
                      found = 1;
                      strncpy(pErrMsg->Body,s+1,MAXFIELDLEN);
                      pErrMsg->Body[MAXFIELDLEN-1] = 0;

                      if ( pErrMsg->Body[strlen(pErrMsg->Body)-1] == '\n' )
		      {
                        pErrMsg->Body[strlen(pErrMsg->Body)-1] = 0;
                      }
                    }
                  }
                } /* fin du while */
              }
              else
	      {
                ReturnCode = -1;
                printf("Hermes - ErrSend() : %s : format incorrect\n",errfilepath);
              }
            }
            else
	    {
              ReturnCode = -1;
              printf("Hermes - ErrSend() : %s : fichier vide\n",errfilepath);
            }

            fclose(f);
          }
          else
	  {
            ReturnCode = -1;
            printf("Hermes - ErrSend() : Impossible d'ouvrir le fichier \"%s\"\n",errfilepath);
          }
        } /* fin du if pErrMsg->ClassPackage[0] != 0 */
      } /* fin du if ErrCode != -1 */

      ErrDispatch(pErrMsg);

      free( pErrMsg );

    } /* fin de pErrMsg != NULL */
  } /* fin de config bien initialisee */

  return( ReturnCode ); /* on retourne 0 si OK, -1 sinon */
}

int ErrGet ( int ErrCode, char * Buffer, int BufSize )
{
  int ReturnCode = 0;
  T_ErrMsg * pErrMsg; /* pointeur sur message d'erreur */
  T_ClassInfo * pCfg; /* pointeur sur structure d'info de classe */
  char errfilepath[MAXFILEPATHLEN]; /* nom complet du fichier des erreurs */
  int len = 0;
  FILE * f;
  char line[512]; /* buffer pour ligne du fichier */
  char * s1;
  char * s2;
  char SysErrMsg[MAXFIELDLEN];
  int AppCode;
  char AppName[MAXNAMELEN];

  memset( (void *)Buffer, 0, BufSize );

  strncpy(SysErrMsg,strerror(errno),MAXFIELDLEN); /* recupere message erreur systeme */

  SysErrMsg[MAXFIELDLEN - 1] = 0;

  if ( SysErrMsg[( strlen(SysErrMsg) - 1 )] == '\n' )
  {
    SysErrMsg[( strlen(SysErrMsg) - 1 )] = 0;
  }

  pthread_once( &once, ErrMutexCreate ); /* creation de mutex une seule fois */

  ErrMutexLock(); /* lock la mutex du package Hermes */

  pCfg = ErrCfgGet(); /* recuperation du pointeur sur la config */

  if ( pCfg == NULL ) /* si la config n'est pas initialisee */
  {
    ErrCfgInit();
  }

  pCfg = ErrCfgGet(); /* recuperation du pointeur sur la config */

  AppCode = GetHermesApplicationCode();

  strncpy(AppName, GetHermesApplicationName() , MAXNAMELEN - 1 );

  AppName[ MAXNAMELEN - 1 ] = 0;

  ErrMutexUnlock(); /* unlock la mutex */

  if ( pCfg == NULL ) /* si la config n'est toujours pas initialisee */
  {
    ReturnCode = -1;
    printf("Hermes - ErrGet() : Probleme a l'init de la config du package Hermes\n");
  }
  else /* on ne continue que si la config est bien initialisee */
  {
    pErrMsg = (T_ErrMsg *)malloc( sizeof(T_ErrMsg) );

    if ( pErrMsg == NULL )
    {
      ReturnCode = -1;
      printf("Hermes - ErrGet() : Memoire insuffisante\n");
    }
    else
    {
      memset((void *)pErrMsg,0,sizeof(T_ErrMsg));

      pErrMsg->ApplicationCode = AppCode;

      strcpy(pErrMsg->ApplicationName,AppName);

      ErrDate(&pErrMsg->Date[0]);

      pErrMsg->ErrCode = ErrCode;

      pErrMsg->ProcessId = getpid();

#if ( _POSIX_VERSION >= 199309L )
      pErrMsg->ThreadId = pthread_self();
#else
      pErrMsg->ThreadId = (pthread_t)0;
#endif

      if ( gethostname(pErrMsg->CpuName,MAXNAMELEN-1) == 0 )
      {
        pErrMsg->CpuName[MAXNAMELEN-1] = 0;
      }
      else
      {
        ReturnCode = -1;
        printf("Hermes - ErrGet() : gethostname() fails !\n");
      }

      if ( ErrCode == -1 )
      {
        memcpy( pErrMsg->Body, SysErrMsg , MAXFIELDLEN );
      }
      else
      {
        pErrMsg->Body[0] = 0;

        pErrMsg->ErrClass = ErrCode >> 16;

        while ( pCfg != NULL )
        {
          if ( pCfg->ClassCode == pErrMsg->ErrClass )
	  {
            strcpy(pErrMsg->ClassPackage, pCfg->PkgName);
            strcpy(errfilepath, pCfg->FilePath);
          }

          pCfg = pCfg->Next;
        }

        if ( pErrMsg->ClassPackage[0] != 0 )
	{
          f = (FILE *)fopen( errfilepath, "r" );

          if ( f != NULL )
	  {
            if ( fgets(line,512,f) != NULL )
	    {
              char ident[20];

              sprintf(ident,"!$%x",pErrMsg->ErrClass);

              if ( strncmp(ident,line,strlen(ident)) == 0 )
	      {
                int found = 0;
                int num;
                char * s;

                while ( (fgets(line,512,f) != NULL) && (found == 0) )
	        {
                  if ( line[0] == '.' )
		  {
                    num = (int)strtol(line+1,&s,16);

                    if ( num == ErrCode )
		    {
                      found = 1;
                      strncpy(pErrMsg->Body,s+1,MAXFIELDLEN-1);
                      pErrMsg->Body[MAXFIELDLEN-1] = 0;

                      if ( pErrMsg->Body[strlen(pErrMsg->Body)-1] == '\n' )
		      {
                        pErrMsg->Body[strlen(pErrMsg->Body)-1] = 0;
                      }
                    }
                  }
                } /* fin du while */

                if (!found)
                {
                  sprintf(pErrMsg->Body,"Unknown error number $%x in class %s\n",
                          pErrMsg->ErrCode,pErrMsg->ClassPackage);
                }
              }
              else
	      {
                ReturnCode = -1;
                printf("Hermes - ErrGet() : %s : format incorrect\n",errfilepath);
              }
	    }
            else
	    {
              ReturnCode = -1;
              printf("Hermes - ErrGet() : %s : fichier vide\n",errfilepath);
            }

            fclose(f);
          }
          else
	  {
            ReturnCode = -1;
            printf("Hermes - ErrGet() : Impossible d'ouvrir le fichier \"%s\"\n",errfilepath);
          }
        } /* fin du if pErrMsg->ClassPackage[0] != 0 */
        else /* pErrMsg->ClassPackage[0] == 0 */
        {
          sprintf(pErrMsg->Body,"Unknown class number %d in error code $%x\n",
                  pErrMsg->ErrClass,pErrMsg->ErrCode);
        }
      } /* fin du if ErrCode != -1 */

      if (Buffer)
      {
        strncpy( Buffer, pErrMsg->Body, BufSize - 1 );

        Buffer[ BufSize - 1 ] = 0;
      }

      free( pErrMsg );

    } /* fin de pErrMsg != NULL */
  } /* fin de config bien initialisee */

  return( ReturnCode ); /* on retourne 0 si OK, -1 sinon */
}

static int ErrDispatch ( T_ErrMsg * pErrMsg )
{
  int ReturnCode = 0;
  char * Buffer;
  char * LoggerBuffer;
  int sev;

  if ( pErrMsg == NULL )
  {
    printf("Hermes - ErrDispatch() : internal error (NULL pointer)\n");
    ReturnCode = -1;
  }
  else
  {
    Buffer = (char *)malloc( sizeof(T_ErrMsg) );

    if ( Buffer == NULL )
    {
      ReturnCode = -1;
      printf("Hermes - ErrDispatch() : Memoire insuffisante\n");
    }
    else
    {
      memset( (void *)Buffer, 0, sizeof(T_ErrMsg) );

/*
      sprintf(Buffer,"==== HERMES MESSAGE ====\nCPU : %s\nProcess ID : %d\nThread ID : %d\nCalling Package : %s\nClass : %s (%d)\nCode : $%x\nType : $%x\nSource : %s\nBody : %s\nExtra : %s\n========================\n",
              pErrMsg->CpuName,
              pErrMsg->ProcessId,
              pErrMsg->ThreadId,
              pErrMsg->CallingPackage,
              pErrMsg->ClassPackage,
              pErrMsg->ErrClass,
              pErrMsg->ErrCode,
              pErrMsg->ErrType,
              pErrMsg->Source,
              pErrMsg->Body,
              pErrMsg->ExtraInfo );
*/
      sprintf(Buffer,"==== HERMES MESSAGE ==== %s\nApplication : %s (%d), Class : %s (%d), Code : $%x, Type :",
              pErrMsg->Date,
              pErrMsg->ApplicationName,pErrMsg->ApplicationCode,
              pErrMsg->ClassPackage,
              pErrMsg->ErrClass,
              pErrMsg->ErrCode);

      if ( pErrMsg->ErrType & HERMES_DEBUG )   strcat(Buffer," DEBUG");
      if ( pErrMsg->ErrType & HERMES_INFO )    strcat(Buffer," INFORMATION");
      if ( pErrMsg->ErrType & HERMES_WARNING ) strcat(Buffer," WARNING");
      if ( pErrMsg->ErrType & HERMES_ERROR )   strcat(Buffer," ERROR");
      if ( pErrMsg->ErrType & HERMES_FATAL )   strcat(Buffer," FATAL");

      sprintf(Buffer+strlen(Buffer),"\nSource : %s\nBody : %s\nExtra : %s\n========================\n",
              pErrMsg->Source,
              pErrMsg->Body,
              pErrMsg->ExtraInfo );

      printf("%s\n", Buffer );

      if ( SendToLogger == HERMES_SEND ) /* s'il faut envoyer au Logger */
      {
        if ((LoggerBuffer = (char *)malloc(sizeof(T_ErrMsg))) == NULL)
        {
          ReturnCode = -1;
          printf("Hermes - ErrDispatch() : Memoire insuffisante\n");
        }
        else
        {
          sprintf(LoggerBuffer,"%s - %s - %s",pErrMsg->Source,pErrMsg->Body,pErrMsg->ExtraInfo);

          if      (pErrMsg->ErrType & HERMES_CATASTROPHE) sev = MSG_S_CATASTROPHE;
          else if (pErrMsg->ErrType & HERMES_FAILURE)     sev = MSG_S_FAILURE;
          else if (pErrMsg->ErrType & HERMES_FAULT)       sev = MSG_S_FAULT;
          else if (pErrMsg->ErrType & HERMES_ALARM)       sev = MSG_S_ALARM;
          else if (pErrMsg->ErrType & HERMES_FATAL)       sev = MSG_S_FAILURE;
          else if (pErrMsg->ErrType & HERMES_ERROR)       sev = MSG_S_ERROR;
          else if (pErrMsg->ErrType & HERMES_WARNING)     sev = MSG_S_WARNING;
          else if (pErrMsg->ErrType & HERMES_INFO)        sev = MSG_S_INFORMATION;
          else if (pErrMsg->ErrType & HERMES_DEBUG)       sev = MSG_S_DEBUG;
          else                                            sev = 0;

          msgDefineMessage(0,pErrMsg->ApplicationCode,sev,NULL,LoggerBuffer);

          free(LoggerBuffer);
        }
      }

      free(Buffer);
    }
  }

  return( ReturnCode );
}
