/***************************************************************/
/*                                                             */
/*                Package Hermes                               */
/*                                                             */
/* HermesCompile.c                                             */
/*                                                             */
/* Copyright GANIL, Groupe Informatique Physique, 1998         */
/*                                                             */
/* Auteur : Frederic Saillant                                  */
/*                                                             */
/* Date de creation : 3 mars 1998                              */
/*                                                             */
/* Derniere mise a jour : 3 avril 1998                         */
/*                                                             */
/* Compilateur de fichiers de description de messages d'erreur */
/***************************************************************/

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define MAXNAMELEN 40
#define MAXTEXTLEN 256

void usage ( void );

void usage ( void )
{
  printf("\nUsage : HermesCompile filename\n");
  printf("\n        avec filename = nom du fichier a compiler\n");
}

void main ( int argc, char ** argv )
{
  FILE * SrcFile = NULL;
  char Line[512];
  char * s1;
  char PkgName[MAXNAMELEN];
  FILE * HFile = NULL;
  FILE * ObjFile = NULL;
  char * s2;
  int ClassCode = -1;
  char HFileName[MAXNAMELEN+8];
  char ObjFileName[MAXNAMELEN+8];
  char ErrorName[MAXNAMELEN]; /* nom d'une erreur */
  char MacroName[2*MAXNAMELEN+3]; /* nom de la macro de l'erreur */
  int LineNum = 0; /* numero de ligne dans le fichier source */
  int ErrorNum = 0; /* numero d'erreur dans la classe */
  char ErrorText[MAXTEXTLEN]; /* texte d'une erreur */

  PkgName[0] = 0;

  if ( argc == 1 )
  {
    usage();
    exit(0);
  }

  SrcFile = fopen( argv[1], "r" );

  if ( SrcFile == NULL )
  {
    printf("\nLe fichier \"%s\" est introuvable\n", argv[1] );
    exit(0);
  }

  while ( fgets(Line,512,SrcFile) != NULL )
  {
    LineNum++; /* incrementation du numero de ligne */

    if ( strncmp(Line, ".CLASS", 6) == 0 )
    {
      /* on ferme les fichiers generes si ils etaient ouverts */
      if ( HFile != NULL )
      {
        fprintf(HFile,"#endif\n");
        fclose( HFile );
      }

      if ( ObjFile != NULL )
      {
        fclose( ObjFile );
      }

      s1 = &Line[6];

      while ( (*s1 == ' ') || (*s1 == '\t') ) s1++; /* on saute les blancs */

      if ( (*s1 == 0) || (*s1 == '\n') )
      {
        printf("\nLigne %d : %s\nIl manque le nom de classe\n",LineNum,Line);
        fclose( SrcFile );
        exit(0);
      }

      strncpy(PkgName,s1,MAXNAMELEN);

      PkgName[MAXNAMELEN-1] = 0;

      s2 = PkgName;

      while ( (*s2 != 0) && (*s2 != ' ') && (*s2 != '\t') )
      { /* on passe en majuscules */
        if ((*s2 >= 'a') && (*s2 <= 'z')) *s2 = *s2 - ('a' - 'A');
        s1++;
        s2++;
      }

      *s2 = 0;

      while ( (*s1 == ' ') || (*s1 == '\t') ) s1++; /* on saute les blancs */

      if ( (*s1 == 0) || (*s1 == '\n') )
      {
        printf("\nLigne %d : %sIl manque le numero\n",LineNum,Line);
        fclose( SrcFile );
        exit(0);
      }

      ClassCode = atoi(s1);

      sprintf(HFileName,"%s_err.h",PkgName);
      sprintf(ObjFileName,"%s_err.hermes",PkgName);

      HFile = fopen( HFileName, "w" );

      if ( HFile == NULL )
      {
        printf("\nCreation du fichier \"%s\" impossible\n",HFileName);
        fclose( SrcFile );
        exit(0);
      }

      ObjFile = fopen( ObjFileName, "w" );

      if ( ObjFile == NULL )
      {
        printf("\nCreation du fichier \"%s\" impossible\n",ObjFileName);
        fclose( HFile );
        fclose( SrcFile );
        exit(0);
      }

      fprintf(HFile,"#ifndef _%s_ERR_H_\n#define _%s_ERR_H_\n",PkgName,PkgName);

      ErrorNum = 0; /* reset du numero d'erreur dans la classe */

      fprintf(ObjFile,"!$%x\n",ClassCode);
    }
    else
    {
      s1 = Line;

      while ( (*s1 == ' ') || (*s1 == '\t') ) s1++; /* on saute les blancs */

      strncpy(ErrorName,s1,MAXNAMELEN);

      s2 = ErrorName;

      ErrorName[MAXNAMELEN-1] = 0;

      while ( /* caracteres autorises pour le nom de l'erreur */
              ( ( *s2 >= 'A' ) && ( *s2 <= 'Z' ) ) ||
              ( ( *s2 >= 'a' ) && ( *s2 <= 'z' ) ) ||
              ( ( *s2 >= '0' ) && ( *s2 <= '9' ) ) ||
              ( *s2 == '_' )
	    )
      {
        s1++;
        s2++;
      }

      if ( s2 != ErrorName ) /* si on a bien un nom d'erreur */
      {
        if ( ErrorNum == 65535 )
	{
          printf("\nLigne %d : %s\nLe nombre maximum d'erreurs est 65535\n",LineNum,Line);

          if ( HFile != NULL )
	  {
            fprintf(HFile,"#endif\n");
            fclose( HFile );
          }

          if ( ObjFile != NULL )
	  {
            fclose( ObjFile );
          }

          fclose( SrcFile );
          exit(0);
        }

        *s2 = 0; /* on termine la chaine du nom d'erreur */

        if ( PkgName[0] == 0 )
	{
          printf("\nLigne %d : %s\nDeclaration d'une erreur sans declaration prealable de classe\n",
                 LineNum,Line);
          fclose( SrcFile );
          exit(0);
        }

        ErrorNum++; /* incrementation du numero d'erreur dans la classe */

        sprintf(MacroName,"R_%s_%s",PkgName,ErrorName);

        while ( (*s1 == ' ') || (*s1 == '\t') ) s1++; /* on saute les blancs */

        ErrorText[0] = 0; /* texte vide par defaut */

        if ( *s1 == '\"' )
	{
          s1++;

          strncpy(ErrorText,s1,MAXTEXTLEN-1);

          ErrorText[MAXTEXTLEN-1] = 0;

          s2 = ErrorText;

          while ( (*s1 != 0) && (*s1 != '\n') && (*s1 != '\"') )
	  {
            s1++;
            s2++;
          }

          *s2 = 0; /* on termine la chaine du texte de l'erreur */
        }

        fprintf(ObjFile,".%x.%s\n", (ClassCode<<16)+ErrorNum, ErrorText );

        fprintf(HFile,"#define %s 0x%x\n", MacroName, (ClassCode<<16)+ErrorNum );
      }
    }
  }

  /* on ferme les fichiers generes si ils etaient ouverts */
  if ( HFile != NULL )
  {
    fprintf(HFile,"#endif\n");
    fclose( HFile );
  }

  if ( ObjFile != NULL )
  {
    fclose( ObjFile );
  }

  fclose( SrcFile );

  exit(0);
}
