#ifndef __SERVSHMDES__
#define __SERVSHMDES__
/*	ServShmDes.h -- creation date : last update : 16 mars 1999
	Copyright Acquisition group, GANIL Caen, France
	author	: Luigi Martina

 	contents : Definitions de la table des serveurs 

	modifications :
	 - B. Raine le 9 mars 99
             modif table SVCC_CMD_TAB

*/

/* 
   Un serveur sera declare dans un fichier sous la forme :
     SERV, NomService, NomCpu, TimeoutSend, TimeoutAnsw, Protocol, [PortIP]
        Les timeout sont exprimes en seconde

   Dans tous les cas, un serveur dispose d'une queue de messages dont le nom 
   est le nom de service. Cette queue sera creee par le serveur (ou par le
   programme d'init de la table de services ?).

   Pour l'instant, le seul protocol utilise est SVCC. 

   On peut prevoir qu'un serveur puisse aussi avoir un acces TCP/IP direct.
   Dans ce cas, on precisera en parametre optionnel le numero de port IP.

   Meme dans le cas d'un serveur IP, on utilisera les messages queues pour
   communiquer entre les threads TCP/IP et le thread executant, puisqu'on 
   devra etre multi-threads pour autoriser le multiclient.

*/


/* definitions de quelques valeurs par defaut */

#define LNAM_SERV	 32

#define LNAM_MSGQ        48

#define LNAM_SEM	 48

#define LNAM_SHM  	 32

#define MAX_SERV_NB	 64

#define MAX_MQ_NB	 64

#define MAX_SVCC_SERV_NB 32
#define MAX_SVCC_CMD_NB	 64

#define SIG_MSGQ_NOTIFY	 SIGRTMIN

#define SIG_MSGQ_PURGE	 SIGRTMAX

#define SIG_SERV_DEBUG   SIGRTMAX -1

#define SIG_SERV_QUIT    SIGRTMAX -2

#define SIG_SERV_SURV	 SIGRTMAX -4

#define SIG_SERV_START   SIGRTMAX -3

#define SIG_MSGQ_DATA	 0

#define MSGQ_SIZE	 2048

#define MSGQ_MAX_MSG	 5

#define MSGQ_PRIO_MIN	 0

#define MSGQ_PRIO_MAX	 MQ_PRIO_MAX - 1

#define SERV_SHM_NAME    "/ServShm1"

#define SEM_PATTERN 	"Sem"

#define MQ_PATTERN 	"Mq"

#define FLAG_BUSY	1

#define FLAG_FREE	0

#ifndef PAGE_SIZE
#define PAGE_SIZE 	4096
#endif

#ifndef TRUE
 #define TRUE 1
#endif

#ifndef FALSE
 #define FALSE 0
#endif

#define PERMISSION (mode_t)0666	  /* set file permission to Read/Write by everyone */

#ifndef EXIT_SUCCESS
 #define EXIT_SUCCESS 0
#endif
 
#ifndef EXIT_FAILURE
 #define EXIT_FAILURE -1
#endif


#ifndef _D
 #define _D *
#endif


/* definition protocole SVCC pour les serveurs */

#define SERV_PROTOCOL_SVCC 	1

#define SERV_PROTOCOL_NONE	0



/* Description Message Queue Associee a chaque serveur */
typedef struct ServMqEntry 
{
  char   MqName[LNAM_MSGQ];     /* Nom de la queue */
  char   MqSemName[LNAM_SEM];  /* Semaphore d'acces en ecriture a la queue */
  int    MessSize;              /* Taille message en bytes */
  int    MessMaxNum;            /* Nombre maximum de messages dans la queue */
  pid_t  Pid;                   /* Process Id du proprietaire de la queue */
  tid_t  Tid;
  int    Ptid;
  int    SigNum;                /* Numero d'evenement */
  int    SigVal;                /* Valeur associee a ce signal */
  int    Prio;
  int    FlagBusy;
  int    free[10];
} MQ_ENTRY;


/* Description Parametres Message Queue Associee a chaque serveur */

typedef struct MqParam 
{
  char   MqName[LNAM_MSGQ];     /* Nom de la queue */
  int    MessSize;              /* Taille message en bytes */
  int    MessMaxNum;            /* Nombre maximum de messages dans la queue */
  int    SigNum;                /* Numero d'evenement */
  int    SigVal;                /* Valeur associee a ce signal */
  int    Prio;
} MQ_PARAM;



/* Description d'une entree serveur SVCC */

typedef struct SvccServ
{
  char ServName[LNAM_SERV];      /* Nom du serveur */
  int  Protocol;
  int  TimeoutSend;              /* en tics de 10 milli-secondes */
  int  TimeoutAnsw;              /* en tics de 10 milli-secondes */
  char CpuName[LNAM_SERV];       /* Nom du cpu contenant le serveur 
				    (0 si local) */
  char CpuIpName[LNAM_SERV];     /* Nom IP du cpu contenant le serveur 
				    (0 si local) */
} SVCC_SERV_ENTRY;

/* Description d'une entree Nom de commande SVCC */

typedef struct SvccCmd
{
  char SvccCmdName[LNAM_SERV];	 /* Nom de Commande */
  int  SvccServEntryNum;         /* Entry number in SVCC server table */
} SVCC_CMD_ENTRY;


/* Description de la table des commandes et serveurs SVCC */

typedef struct SvccCmdTab
{
  SVCC_SERV_ENTRY SvccServEntry[MAX_SVCC_SERV_NB];
  SVCC_CMD_ENTRY  SvccCmdEntry[MAX_SVCC_CMD_NB];
} SVCC_CMD_TAB;





/* definition structure info CPU, */
/* en cas d'acces via TCP/IP      */

typedef  struct CpuInfo{
      char CpuName[LNAM_SERV];  /* Nom IP du Cpu contenant le serveur */
      int  IpPort;              /* Numero de port */
 } CPU_INFO;
 





/* Description d'une entree serveur */

typedef struct ServEntry
{
  char ServName[LNAM_SERV]; 	/* Nom du serveur */
  int  ServProtocol;		/* Protocol SVCC ou ...	 */
  int  ServAccessMode;		/* Mode d'acces au serveur */ 
  union {
    CPU_INFO CpuInfo;		/* stucture CpuInfo */
    int MqNumEntry;		/* num entree MqTab */ 
  } Access;
  int Debug;
  int FlagBusy;
  int free[19];
} SERV_ENTRY;


/* Description de la table des serveurs */
typedef struct ServTab
{
  int ServNumber;             /* Nombre de serveurs dans la table */
  SERV_ENTRY ServEntry[MAX_SERV_NB];
} SERV_TAB;


/* Description de la table des messages queue */

typedef struct MqTab
{
  int MqNumber;               /* Nombre de queues de messages dans la table */
  MQ_ENTRY MqEntry[MAX_MQ_NB];

} MQ_TAB;


/* Description de la memoire des services */

typedef struct
{
  char SemName[LNAM_SEM];
  SVCC_CMD_TAB SvccCmdTab;
  SERV_TAB ServTab;
  MQ_TAB MqTab;
} SERV_MEMORY;

#define SERV_SHM_SIZE ((sizeof(SERV_MEMORY) & 0x0fffff000) + PAGE_SIZE)






/* prototypage des fonctions d'utilite publique */

int ShmMap(char *ServShmName, int Size, int Prot, char **LogAddr); 

int ShmUnMap(char *LogAddr, int Size);

int ShmCreate(char *ShmName);

int ShmRemove(char *ShmName);

int ServShmInit(char *ServShmName, int MemSize);

int ServCreate(char *ServName, int Protocol, int ServAccesMode, MQ_PARAM *MqParam);

int ServRemove(char *ServName);

int SemCreate(char *SemName,int value);

int SemPost(char *SemName);

int SemWait(char *SemName, int Timeout);

int SemRemove(char *SemName);

int InitMqEntry(MQ_ENTRY *MqEntry ,MQ_ENTRY *MqParam);

int MqCreate(char *Mqname, int MqMessSize, int MqMessNb);

int MqReceive(char *Mqname, char *Mess, int MessSize,int *Prio );

int MqSend(char *Mqname, char *Mess, int MessSize,int Prio );

int RazMqEntry(MQ_ENTRY *MqEntry);

int RazServEntry(SERV_ENTRY *ServEntry);

int InitServEntry(SERV_ENTRY *ServEntry, SERV_ENTRY *ServParam);

int RazSvccEntry(SVCC_CMD_ENTRY *SvccCmdEntry);

int ServNameToSemName(char *ServName, char *SemName);

int ServNameToMqName(char *ServName, char *MqName);

int ServShmCreate(char *ServShmName);

int MajServMem(SERV_MEMORY *ServShm);

int MqNameToServName(char *MqName, char *ServName);

int SemNameToServName(char *SemName, char *ServName);

int ListServer(void);

int ShmNameToSemName(char *ShmName, char *SemName);

int GetServInfoByName(char *ServName, SERV_ENTRY *ServEntry,MQ_ENTRY *MqEntry);

int GetServInfoByPtid(int Ptid, SERV_ENTRY *ServEntry,MQ_ENTRY *MqEntry);

int GetServInfoByMqName(char *MqName, SERV_ENTRY *ServEntry,MQ_ENTRY *MqEntry);

int GetNewServEntry(SERV_MEMORY *ServShm, char *ServName, int *ServNumEntry, int *MqNumEntry);int GetServInfoByName(char *ServName, SERV_ENTRY *ServEntry,MQ_ENTRY *MqEntry);

int GetServInfoByNum(int ServNumEntry, SERV_ENTRY *ServEntry,MQ_ENTRY *MqEntry);

#endif
