# 
#    remote command interface
#    intended for use by the VAMOS control with EXOGAM
#

set frame "Remote"
set frametitle "EXOGAM remote control interface"
set icontitle "Remote"


set IPaddress 0
set TCPport 0
set MyPort 10260

set Application "local"

proc do_listen {} {
   global MyPort IP

    set rc [catch {set IP(DummySock) [socket -server do_connected $MyPort]} msg]
    if { $rc != 0 } {
               insert "$msg"
               return 0
    }
    insert "Socket created with ID= $IP(DummySock)"
}

proc do_disconnect {} {
    global IP
    
    if {[info exists IP(DummySock)]} then {
        close $IP(DummySock)
        insert "Socket $IP(DummySock) closed"
        unset IP(DummySock)
    }
    
   if {[info exists IP(DataSock)]} then {
        close $IP(DataSock)
        insert "Socket $IP(DataSock) closed"
        unset IP(DataSock)
    }
}

proc do_connected {socket address port} {
    global IP IPaddress TCPport Application

    insert "Connection received on socket $socket from IP address $address TCP port $port"
    
    set IPaddress $address
    :modify IPADDR :value $IPaddress
    set TCPport $port
    :modify TCPPORT :value $TCPport
    
    set Application "local"
    
# If the new channel is accepted, the dummy channel will be closed
    catch {close $IP(DummySock); unset IP(DummySock)}

    set IP(DataSock) $socket

# Configure translation mode
    fconfigure $socket -buffering line -blocking 1

# Set event notification when data arrives
    fileevent $socket readable do_command
}

proc do_command {} {
    global env IP global Command Application
    
    set length [gets $IP(DataSock) Command]
    if {$length < 0} {
    
        if {[eof $IP(DataSock)] == 1} then {

#    eof condition exists
            insert "Socket $IP(DataSock) has disconnected"
            
            do_disconnect
            do_listen
            return
        }
    }
    
    if {$length <= 0} then {
    
# Set event notification when data arrives

        fileevent $IP(DataSock) readable do_command
        return
    }
    
#    a full line of input is available without blocking

    set valid 0
    set c [string range $Command 0 0]
    if {$c == "#"} then {
        insert "[string range $Command 1 end]"
        incr valid
    }
    if {$c == "%"} then  {
        set tasks [linsert [Get_Task_Names] 0 local]
        set new [string range $Command 1 end]
        if {[lsearch $tasks $new] < 0} then {
            insert "Invalid target $new"
            puts $IP(DataSock) "#Invalid target $new"
        } else {
            set Application $new
            insert "Target now $Application"
        }
        incr valid
    }
    if {$c == "*"} then {
        set env(FooterText) ""
        set env(ErrorText) ""
        set env(WarningText) ""
        set env(ReportText) ""
        if {$Application == "local"} {
            set env(EGsilent) 1
            set result [catch [list uplevel #0 [string range $Command 1 end]] msg]
            set env(EGsilent) 0
            puts $IP(DataSock) $msg
        } else {
            set env(EGsilent) 1
            set result [catch {inform $Application [string range $Command 1 end]} msg]
            set env(EGsilent) 0
            puts $IP(DataSock) "$msg\n|$env(ErrorText)\n|$env(WarningText)\n|$env(ReportText)\n|$env(FooterText)"
        }
        incr valid
    }
    if {$valid == 0} then {
        insert "Invalid command received: $Command"
        puts $IP(DataSock) "#Invalid command $Command"
    }

# Set event notification when data arrives
    fileevent $IP(DataSock) readable do_command
}


proc click_ipaddr {vl} {global IPaddress; set IPaddress $vl}
proc click_tcpport {vl} {global TCPport; set TCPport$vl}
proc click_myport {vl} {global MyPort; set MyPort $vl}

proc make-frame {} {
  global env MyPort IPaddress TCPport
  
set rows 10
set cols 90

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 10

      :text MYPORT    :label "Local TCP Port:"     :value $MyPort :w 6
      :text IPADDR    :label "Remote IP Address:"  :value $IPaddress :w 16
      :text TCPPORT   :label "Remote TCP Port:"    :value $TCPport   :w 6

      :button LISTEN     :label "Listen for connection"  :command do_listen
      :button DISCONNECT :label "Disconnect"             :command do_disconnect

   :panel P3 :layout T
       :galley LIST :rows $rows :cols $cols :deaf T  :font variable
    
   open-frame

  :galley_size LIST $rows $cols
}

proc click-configure {W} {:galley_resize LIST}

proc insert {s} {:modify LIST :strings $s\n}

proc start args {
   global started
   if {$started == 0} then {
      make-frame
      set started 1
   } else {open-frame}
}

