proc Make_Configuration_File {} {
    global env EGreport CurrentVXIServer
    global VXICrateList

#    obtain list of all data sources

   if {![midas-confirmation " Continuing will update the information file used by the data processing and analysis software.\n\
     If you have changed the configuration of VXI modules this must be updated."]} then {return}

   set Configdir [file join $env(MIDASAPPLBASE) config]
   set experiment [file tail $env(EXPERIMENTBASE)]
   set fname [file join $Configdir [set experiment]_Current_VXI_Configuration]

   set z [catch {open $fname w} f]
   if {$z != 0} then {midas-warning "Unable to create file $fname\n$f"; return}

#BR. Add a # before name in title line
   puts $f "[format "%-24s%-10s%-10s%-10s%-10s%-10s" \#name group item label enable databits]"

   set EGreport_BAK $EGreport
   set EGreport -1

   set-footer "Obtaining list of all known data sources from the VXI servers"

 foreach crate $VXICrateList {
   set CurrentVXIServer [VXIcrate2server $crate]

#    get RM and VRE numbers
   set rm   [first [split [ersinquire RM*.VXI.Passed] .]]
   set vre  [first [split [ersinquire VRE*.VXI.Passed] .]]

#    and all the data sources 
   set nameslist [ersinquire *.Addr]

   set-footer "Creating configuration information in file $fname"

   puts $f "#crate $crate $rm $vre"

   foreach name $nameslist {
      if {[second [split $name .]] == "GIR"} then continue
      if {[second [split $name .]] == "Global"} then continue

#     for this data source obtain address information 
      set address [rr $name]
      set group [expr $address & 255]
      set item [expr $address >> 8]
      set label [expr $address]

#    remove the trailing .Addr to obtain name of data source 
      set name [string range $name 0 [expr [string length $name] - 6]]

      switch -glob [first [split $name .]] {
          Saphir*   {set module Saphir}
          HRGe8x*   {set module HRGe8x}
          ECC*      {set module ECC}
          ESS*      {set module ESS}
          GOCCE*    {set module GOCCE}
          CsI*      {set module CsI}
          ADC6414V* {set module ADC6414V}
          ADC3214V* {set module ADC3214V}
          TDC3214V* {set module TDC3214V}
          QDC3214V* {set module QDC3214V}
          TriggerMK2 {set module TriggerMK2}
          Centrum   {set module Centrum}
          default   {set-footer "module type not supported"; continue}
      }

#    obtain if enabled or not
      set enable 0
      switch $module {
          Saphir    -
          CsI       {set enable [rr [first [split $name .]].Enable]}
          HRGe8x    {set enable [rr [first [split $name .]].[second [split $name .]].Enable]}
          ECC       {set enable [rr [first [split $name .]].[second [split $name .]].Enable]}
          GOCCE     {set enable [rr [first [split $name .]].[second [split $name .]].[third [split $name .]].Enable]}
          ESS       {set enable [rr [first [split $name .]].CCR.Enable]}
          ADC6414V  {set enable [rr $name.Enable]}
          ADC3214V  {set enable [rr $name.Enable]}
          TDC3214V  {set enable [rr $name.Enable]}
          QDC3214V  {set enable [rr $name.Enable]}
          TriggerMK2  {set enable [rr TriggerMK2.RdOut.[second [split $name .]].Enable]}
          Centrum   {set enable [rr Centrum.RdOut.[second [split $name .]].Enable]}
      }
#    if there is any reply other than 1 (enabled) then set the data source to be not enabled
      set enable [expr $enable]
      if {$enable != 1} then {set enable 0}

#    set as always enabled until we can think of something better to do!!!  -   user can change while GOING
      set enable 1 

#    finally require the number of data bits 
      switch $module {
         CsI      -
         Saphir   {set databits 13}
         ESS      {set databits 12}
         HRGe8x   {
# modif 25/04/04 Ch. Theisen : 15 bits for TAC
	(it is 16 bits, but ... only 15 bits for ganil acquisition)
	 	switch $item {
			0 {set databits 14}
			1 {set databits 14}
			2 {set databits 15}
			3 {set databits 14}
			4 {set databits 14}
			5 {set databits 15}
			6 {set databits 14}
			7 {set databits 14}
			8 {set databits 15}			
			9 {set databits 14}
			10 {set databits 14}
			11 {set databits 15}			
		}
	 }
# modif 25/04/04 Ch. Theisen : 15 bits for TAC	 
         ECC      {
	 	switch $item {
			0 {set databits 14}
			1 {set databits 14}
			2 {set databits 15}
			3 {set databits 14}
			4 {set databits 14}
			5 {set databits 15}
			6 {set databits 14}
			7 {set databits 14}
			8 {set databits 15}			
			9 {set databits 14}
			10 {set databits 14}
			11 {set databits 15}	
		}
	 }
         GOCCE    {set databits 14}
         ADC6414V {set databits 14}
         ADC3214V {set databits 14}
         TDC3214V {set databits 14}
         QDC3214V {set databits 14}
         TriggerMK2 {set databits 16}
         Centrum   {set databits 16}
      }

#      puts $f "[format "%-25s%-10s%-10s%-10s%-10s%-10s" $name $group $item $label $enable $databits]"
# changed 14/4/4 Ch. Theisen
      puts $f "[format "%-26s%-10s%-10s%-10s%-10s%-10s" $name $group $item $label $enable $databits]"
    }

#    now do the Centrum which is a special case at present

    set nameslist [ersinquire Centrum0.label.*]

    foreach name $nameslist {

#     for this data source obtain address information 
      set address [rr $name]
      set group [expr $address & 255]
      set item [expr $address >> 8]
      set label [expr $address]

#    remove the trailing .Addr to obtain name of data source

      set name [split $name .]
      set name [first $name].[third $name].[fourth $name]

      set enable 1
      set databits 16

#      puts $f "[format "%-25s%-10s%-10s%-10s%-10s%-10s" $name $group $item $label $enable $databits]"
# changed 14/4/4 Ch. Theisen
      puts $f "[format "%-26s%-10s%-10s%-10s%-10s%-10s" $name $group $item $label $enable $databits]"
    }

  }

    close $f
    set-footer "$fname created"

    set EGreport $EGreport_BAK
}

proc rr {r}  {
   global CurrentVXIServer EGallow  EGallowAll

  set EGallow $EGallowAll
  set vl [EG "register :name '/$CurrentVXIServer/$r' ?value"]
  set EGallow 0

   return $vl
}
