#    d2vbdiag.tcl  -  D2VB module diagnostic frame


set frame d2vbdiag
set frametitle "D2VB Diagnostics Frame"
set icontitle "D2VBdiag"

set mname "D2VB"

if {[info exists env(D2VBserver)]} then {set resource $env(D2VBserver)} \
    else {set resource VME}

#    load the common f2vb/d2vb source

loadsource x2vbdiag

set DataWidth 32

#    for the formatted dump
set ECCBase 160
set ECCLimit 174
set ESSBase 128
set ESSLimit 147
set CsIBase 191
set CsILimit 205
set ADC64Base 151
set ADC64Limit 152
set QDC64Base 153
set QDC64Limit 154
set ADC32Base 111
set ADC32Limit 115
set TDC32Base 121
set TDC32Limit 125
set QDC32Base 116
set QDC32Limit 120
set GOCCEBase 175
set GOCCELimit 190



set F2VB_registerbase 0x00e00000
set F2VB_listrambase  0x00e00800

proc define-x2vb-registers {} {
   global F2VB_registerbase F2VB_listrambase
   global resource mname dumpwords

   set register /$resource/$mname.D*
   set-footer "deleting register $register"
   EG "register :pattern '$register' !delete"

   define-register xVME DCSR  "-a $F2VB_registerbase  -o 0   -w 24 -d 16 -z x -p rw"
   define-register xVME DCCR  "-a $F2VB_registerbase  -o 2   -w 24 -d 16 -z x -p rw"
   define-register xVME DICR  "-a $F2VB_registerbase  -o 6   -w 24 -d 16 -z x -p rw"
   define-register xVME DIVR  "-a $F2VB_registerbase  -o 8   -w 24 -d 16 -z x -p rw"
   define-register xVME DLRAR "-a $F2VB_registerbase  -o 10  -w 24 -d 16 -z x -p ro"
   define-register xVME DDRAR "-a $F2VB_registerbase  -o 12  -w 24 -d 16 -z x -p rw"
   define-register xVME DCBRA "-a $F2VB_registerbase  -o 16  -w 24 -d 32 -z x -p ro"
   define-register xVME DCBWA "-a $F2VB_registerbase  -o 20  -w 24 -d 32 -z x -p ro"

   define-register bVME Dbram "-a [expr [read-register DDRAR] << 16] -w 32 -d 32 -z x -p rw -l $dumpwords"
   define-register bVME Dlram "-a $F2VB_listrambase   -w 24 -d 16 -z x -p rw -l 96"
}

proc define-register {class name attributes} {
   global resource mname

   set register /$resource/$mname.$name
   set-footer "defining register $register"
   EG "register :name '$register' :class '$class' !new"
   EG "register :name '$register' :object '$attributes'"
   clear-footer
}


proc add-format-option {} {
     :choice-stack FORMATOPT :label "format option" :strings [list 0 1 2] :value 2
}


proc format_the_dump {} {
  global dumpdata
  global dumpopt dumpwords
  global x2VBmemOffset
  global ECCBase ECCLimit
  global ESSBase ESSLimit 
  global ADC64Base ADC64Limit
  global QDC64Base QDC64Limit
  global ADC32Base ADC32Limit
  global TDC32Base TDC32Limit
  global QDC32Base QDC32Limit
  global CsIBase CsILimit
  global GOCCEBase GOCCELimit

#    format dumpdata held in global variable dumpdata

   set-footer "formatting dump"

   set frame "dump"
   :frame [self frame-name].$frame :label  "D2VB Dump"  :transient T

   switch $dumpopt {
   0  {
        :galley G :rows 50 :cols 50 :deaf T :strings "" :font fixed
        :show T
        :modify G :strings "raw dump starting at word offset 0\n"
        :modify G :strings $dumpdata
      }

   1  {
        :galley G :rows 50 :cols 50 :deaf T :strings "" :font fixed
        :show T
        :modify G :strings "dump starting at word offset $x2VBmemOffset for $dumpwords words\n"
        set dump ""
        for {set i 0} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           set group [expr $word >> 16 & 255]
           set item  [expr $word >> 24 & 63]
           if {$group >= 248 && $item == 63} then {
#    last end token for event
             append dump "$word "
             :modify G :strings $dump
             :modify G :strings "\n\n"
             set dump ""
           } else {
             append dump "$word "
           }
        }
        :modify G :strings $dump
     }
   2  {
        :galley G :rows 50 :cols 60 :deaf T :strings "" :font fixed
        :show T
        :modify G :strings "dump starting at word offset $x2VBmemOffset for $dumpwords words\n"
        for {set i 0} {$i < $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           set group [expr $word >> 16 & 255]
           set item  [expr $word >> 24 & 63]
           set qbits [expr $word >> 30 & 3]
           set data  [expr $word & 0x0000ffff]
           set title "Unknown"
           if {$group <= 0} then {
#              invalid
               set title "illegal"
           } elseif {$group >= $ECCBase && $group <= $ECCLimit} then {
#               Exogam cards (ECC)
              set detector [expr $group - $ECCBase]
              set title "ECC"
              switch $item {
              0   {set title "ECC$detector.A.6MeV"}
              1   {set title "ECC$detector.A.20MeV"}
              2   {set title "ECC$detector.A.TAC"}
              3   {set title "ECC$detector.B.6MeV"}
              4   {set title "ECC$detector.B.20MeV"}
              5   {set title "ECC$detector.B.TAC"}
              6   {set title "ECC$detector.C.6MeV"}
              7   {set title "ECC$detector.C.20MeV"}
              8   {set title "ECC$detector.C.TAC"}
              9   {set title "ECC$detector.D.6MeV"}
              10  {set title "ECC$detector.D.20MeV"}
              11  {set title "ECC$detector.D.TAC"}
              default  {}
              }
           } elseif {$group >= $ESSBase && $group <= $ESSLimit} then {
#               Exogam cards (ESS)
              set detector [expr $group - $ESSBase]
              set title "ESS"
              switch $item {
              0   {set title "ESS$detector.Energy"}
              1   {set title "ESS$detector.Pattern"}
              2   {set title "ESS$detector.TDC.QA"}
              3   {set title "ESS$detector.TDC.QB"}
              4   {set title "ESS$detector.TDC.QC"}
              5   {set title "ESS$detector.TDC.QD"}
              6   {set title "ESS$detector.TDC"}
              default  {}
              }
           } elseif {$group >= $GOCCEBase && $group <= $GOCCELimit} then {
#               Exogam cards (GOCCE)
              set detector [expr $group - $GOCCEBase]
              set title "GOCCE"
              switch $item {
              0   {set title "GOCCE$detector.XtalA.Seg0.Energy"}
              1   {set title "GOCCE$detector.XtalA.Seg0.Time"}
              2   {set title "GOCCE$detector.XtalA.Seg1.Energy"}
              3   {set title "GOCCE$detector.XtalA.Seg1.Time"}
              4   {set title "GOCCE$detector.XtalA.Seg2.Energy"}
              5   {set title "GOCCE$detector.XtalA.Seg2.Time"}
              6   {set title "GOCCE$detector.XtalA.Seg3.Energy"}
              7   {set title "GOCCE$detector.XtalA.Seg3.Time"}
              8   {set title "GOCCE$detector.XtalB.Seg0.Energy"}
              9   {set title "GOCCE$detector.XtalB.Seg0.Time"}
              10  {set title "GOCCE$detector.XtalB.Seg1.Energy"}
              11  {set title "GOCCE$detector.XtalB.Seg1.Time"}
              12  {set title "GOCCE$detector.XtalB.Seg2.Energy"}
              13  {set title "GOCCE$detector.XtalB.Seg2.Time"}
              14  {set title "GOCCE$detector.XtalB.Seg3.Energy"}
              15  {set title "GOCCE$detector.XtalB.Seg3.Time"}
              16  {set title "GOCCE$detector.XtalC.Seg0.Energy"}
              17  {set title "GOCCE$detector.XtalC.Seg0.Time"}
              18  {set title "GOCCE$detector.XtalC.Seg1.Energy"}
              19  {set title "GOCCE$detector.XtalC.Seg1.Time"}
              20  {set title "GOCCE$detector.XtalC.Seg2.Energy"}
              21  {set title "GOCCE$detector.XtalC.Seg2.Time"}
              22  {set title "GOCCE$detector.XtalC.Seg3.Energy"}
              23  {set title "GOCCE$detector.XtalC.Seg3.Time"}
              24  {set title "GOCCE$detector.XtalD.Seg0.Energy"}
              25  {set title "GOCCE$detector.XtalD.Seg0.Time"}
              26  {set title "GOCCE$detector.XtalD.Seg1.Energy"}
              27  {set title "GOCCE$detector.XtalD.Seg1.Time"}
              28  {set title "GOCCE$detector.XtalD.Seg2.Energy"}
              29  {set title "GOCCE$detector.XtalD.Seg2.Time"}
              30  {set title "GOCCE$detector.XtalD.Seg3.Energy"}
              31  {set title "GOCCE$detector.XtalD.Seg3.Time"}
              default  {}
              }
           } elseif {$group >= $CsIBase && $group <= $CsILimit} then {
#               Diamant cards (CsI)
              set detector [expr $group - $CsIBase]
              set title "CsI"
              switch $item {
              0   {set title "CsI$detector.Energy"}
              1   {set title "CsI$detector.PI"}
              2   {set title "CsI$detector.Time"}
              default  {}
              }
           } elseif {$group >= $ADC64Base && $group <= $ADC64Limit} then {
#               Exogam cards (ADC6414)
              set title "ADC6414V[expr $group - $ADC64Base].c$item"
           } elseif {$group >= $QDC64Base && $group <= $QDC64Limit} then {
#               Exogam cards (QDC6414)
              set title "QDC6414V[expr $group - $QDC64Base].c$item"
           } elseif {$group >= $ADC32Base && $group <= $ADC32Limit} then {
#               Exogam cards (ADC3214)
              set title "ADC3214V[expr $group - $ADC32Base].c$item"
           } elseif {$group >= $QDC32Base && $group <= $QDC32Limit} then {
#               Exogam cards (QDC3214)
              set title "QDC3214V[expr $group - $QDC32Base].c$item"
           } elseif {$group >= $TDC32Base && $group <= $TDC32Limit} then {
#               Exogam cards (TDC3214)
              set title "TDC3214V[expr $group - $TDC32Base].c$item"
           } elseif {$group == 246} then {
#               trigger
               set title "trigger.item$item"
           } elseif {$group == 247} then {
#               VRE + other diagnostics
               set title "diagnostic"
           } elseif {$group < 248} then {
#               unknown
               set title "unknown group $group"
           } else {
#               tokens 
              if {$item <= 15} then {set title "start token"} \
              elseif {$item <= 31} then {set title "event number lsw"} \
              elseif {$item <= 47} then {set title "event number msw"} \
              else {set title "end token"}
           }
           :modify G :strings "[format "%10s %3d %3d %4d %6x %24s" $word $qbits $item $group $data $title] \n"
           if {$item == 63} then {:modify G :strings "\n"}
       }
     }

   }
   :modify G :see 0
   clear-footer
}
