# singles.tcl--- Singles control frame

# based on hist.tcl
# (explains some redundant code and why it still uses an "overlay")

set debug 0
set started 0

set frame SI
set frametitle "Singles System"
set icontitle "Singles"

#    size of detector overlay panel

set ovlyw 300
set ovlyh 100

#    current singles server resource
set resource ""
#    list of available singles server resources
set resources {}
#    number of available singles server resources
set numresources 0

set sorttype numeric

#    supported detector types
set detector Singles

#    directory selection patterns
set patternmenu {"" *.q0 *.q1 *.q2 *.q3 *.t? *.q0.t? *.q1.t? *.q2.t? *.q3.t? *.q?.t?}
set pattern ""

set lowchannel 0
set highchannel 0
set qualifier 1
set maxchannel 255
set maxgroup 255
set lowitem 0
set highitem 0
set maxitem 63
set modulename ABC

set dirlist {}
set spectrumlist {}

# setup default map of singles names to singles servers
# (items 0 to 7 on server singles1, items 8 to 16 on server singles2)
# then overwrite if optional configuration file exists
for {set i 0} {$i < 16} {incr i} {set Singles($i)  singles[expr $i/8 + 1]}
set SinglesDefn [file join $env(MIDASAPPLBASE) config Exogam SinglesDefn]
if {[file exists $SinglesDefn] == 1} then {source $SinglesDefn}

proc list-take {as is} {
   set ll {}
   foreach e $is {lappend ll [lindex $as $e]}
   return $ll
}

proc prefix-list {l p} {
   set ll {}
   foreach f $l {lappend ll $p$f}
   return $ll
}

#    The section of code which follows handles spectrum server access to
#    create, delete and clear spectra
#    These actions have to be directed to the correct server when there
#    are more than one server in the experiment

set vxicontrolstarted 0

#    procedure to return VXI crate holding a module

proc VXImodule2crate {module} {
  global vxicontrolstarted

#    obtain crate holding a specific module
#    this uses the task  "vxicontrol"

  if {$vxicontrolstarted == 0} then {
     set started [inform vxicontrol inquire started]
     if {$started == 0} then {inform vxicontrol make-frame}
     set vxicontrolstarted 1
  }

  set crate [send vxicontrol VXImodule2crate $module]
  return $crate
}

proc execute-spectrum {n action} {
  global env Singles resource numresources
  global modulename
  global debug

#insert-debug "$action $n"

  if {$numresources <= 1 || ![info exists Singles]} then {set server $resource} else {
#
#    more than 1 server
#    must send action to the appropiate server
#
     set module  [first [split $n .]]
     set group [string range $module 0 [expr [string length $modulename]-1]]
     if {$group == $modulename} then {
        set item [string range $module [string length $modulename] end]
     } else {set item ""}

#    name of global Singles System resource
     if {[info exists env(SinglesServer)] == 0} then {set r "singles"}  \
        else {set r $env(SinglesServer)}

     if {$resource == $r} then {
        if {[info exists Singles($item)]} then {set server $Singles($item)} \
           else {set server $resource}
     }
  }

#insert-debug "$server"

  switch $action {
     create {
        set range [range-of $n]
        if {$debug} then {insert-debug "create spectrum :range $range :name '$n' "}
        set z [catch {eg "(spectrum :path '/$server' :range $range :name '$n' !new)" } m]
#    Allow "Spectrum already exists" so that further spectra can be created
        if {$z!=0 && $m==0x30004} then {set z 0}
     }
     delete {
        if {$debug} then {insert-debug "delete spectrum  :name '$n' "}
        set z [catch {eg "(spectrum :path '/$server' :name '$n' !delete)"} m]
# Allow "Pathname Invalid" so that further spectra can be deleted
        if {$z!=0 && $m==0x30004} then {set z 0}
     }
     clear {
        if {$debug} then {insert-debug "clear spectrum  :name '$n' "}
        set z [catch {eg "(spectrum :path '/$server' :name '$n' !zero)"} m]
     }
  }
  if {$z!=0} then {
    display-eg-error-report self $m "Attempting to " $action "spectrum" $n
    clear-footer
    error $m
  }
}

proc create-spectrum n {execute-spectrum $n create}

proc delete-spectrum n {execute-spectrum $n delete}

proc zero-spectrum n {execute-spectrum $n clear}

proc read-spectra {} {
    global resource pattern dirlist
    if {"$pattern" != ""} then {set path "/$resource/$pattern"} else {set path "/$resource"}
    set z [catch {eg "(spectrum :path '$path' ?names)"} m]
    if {$z!=0} then {
       display-eg-error-report self $m "Attempting to read singles spectra" $path
       clear-footer
       error $m
    }
    set dirlist $m
    sort-dirlist
}

proc compare-numeric {a b} {
   set pat "(\[^0-9\]+)(\[0-9\]+)(.*)(\.*)"
   set za [regexp $pat $a a0 a1 a2 a3 a4]
   set zb [regexp $pat $b b0 b1 b2 b3 b4]
   if {$za==0 || $zb==0} then {return [string compare $a $b]}
   if {"$a"!="$a0" || "$b"!="$b0"} then {return [string compare $a $b]}
   set z [string compare $a1 $b1]
   if {$z!=0} then {return $z}
   set z [expr int($a2.0-$b2.0)]
   if {$z!=0} then {return $z}
   return [string compare $a $b]
}

proc sort-dirlist {} {
   global sorttype dirlist
   set ll {}
   foreach f $dirlist {lappend ll [second $f]\\[first $f]}
   if {"$sorttype"=="numeric"} then \
      {set ll [lsort -command compare-numeric $ll]} \
   elseif {"$sorttype"=="alphabetic"} then \
      {set ll [lsort -ascii $ll]}
   set dirlist {}
   foreach f $ll {
      set s [split $f "\\"]
      lappend dirlist [first $s]
   }
}

proc paint-histlist {} {
   global env dirlist spectrumlist
   :modify LIST :append F :strings $dirlist :see 0
   set spectrumlist {}
}

proc paint-spectrumlist {} {
  global env dirlist spectrumlist
  foreach s $spectrumlist {
    set i [lsearch $dirlist $s]
    if {$i >= 0} then {:modify LIST :select $i}
  }
}

proc click-zero {} {
   global resource
   set c 0
   foreach spectrum [get-spectrum-list] {
      set-footer "zeroing /$resource/$spectrum..."
      zero-spectrum $spectrum
      incr c
   }
   set-footer "total of $c spectra successfully zeroed"
}

proc click-create {} {
   global resource
   set c 0
   foreach spectrum [get-spectrum-list] {
      set-footer "creating /$resource/$spectrum..."
      create-spectrum $spectrum
      incr c
   }
   click-redisplay
   set-footer "total of $c spectra successfully created"
}

proc click-delete {} {
   global resource
   set c 0
   foreach spectrum [get-spectrum-list] {
      set-footer "deleting /$resource/$spectrum..."
      delete-spectrum $spectrum
      incr c
   }
   click-redisplay
   set-footer "total of $c spectra successfully deleted"
}

proc click-show {} {
   global resource
   set list [get-spectrum-list]
#   puts stderr  "show [prefix-list $list /$resource/]"
   inform sd  "show [prefix-list $list /$resource/]"
}

proc click-addresses {} {
        global resource
        set list [prefix-list [get-spectrum-list] /$resource/]
        if {$list=={}} then {
                set-footer "you haven't selected any spectra!!!";
                return
        }
        clear-footer
        foreach sp $list {
                set z [catch {eg "(spectrum  :name '$sp' ?addr)"} m]
                if {$z!=0} then {
                        display-eg-error-report self $m "Attempting to find address of spectrum" $sp
                        clear-footer
                        error $m
                }
                insert-log [transparent [format "%s:\t0x%x" [lindex [split $sp /] 2] $m]]
        }
}

proc click-system d {
   global resource
   set resource $d
   :modify SYSTEM :value $resource
   click-redisplay
}

proc make-resource-list {} {
  global env resource resources numresources

   set z [catch {eg "(experiment ?resources)"} m]
   if {$z==0} then {set rs $m} else {set rs ""}

#    scan through resources and count instances of all singles spectrum servers
   if {[info exists env(SinglesServer)] == 0} then {set role singles}  \
      else {set role $env(SinglesServer)}
   set count 0
   set rolecount(singles) 0
   foreach r $rs {
      set type [second [fourth $r]]
      set role [second [fifth $r]]
      if {$type == "spectrum" && $role == "singles"} then {incr count; incr rolecount(singles)}
   }

   if {[info exists env(SinglesServer)] == 0} then {set resource singles}  \
      else {set resource $env(SinglesServer)}
   set resources {}
   set numresources 0
   if {$rolecount(singles) > 0}   then {lappend resources singles; incr numresources}

   if {$count > 1 || $role != $resource} then {
#    next scan through resources and add instances of all singles spectrum servers to the list
      foreach r $rs {
         set type [second [fourth $r]]
         set role [second [fifth $r]]
         set name [second [first $r]]
         if {$type == "spectrum" && $role == "singles" && $rolecount(singles) > 1}   then {lappend resources $name; incr numresources}
      }
   } else {
      if {[info exists env(SinglesServer)] == 0} then {set resources singles}  \
         else {set resources $env(SinglesServer)}
      set numresources 1
   }
   if {[info exists env(SinglesServer)] == 0} then {set resource [first $resources]}  \
      else {set resource $env(SinglesServer)}

   make-processor-list
}

proc click-list {c s} {
  global spectrumlist
  set spectrumlist $s
  paint-spectrumlist
}

proc click-pattern {p} {
   global  pattern spectrumlist
   :modify PATTERN :value $p
   set pattern $p
   set spectrumlist {}
   click-redisplay
}

proc start-ecc {} {
    set started [inform ecc inquire started]
    if {$started == 0} then {
      send ecc start
      send ecc close-frame
      after 3000
    }
}

proc paint_detector {} {
   global ovlyw ovlyh
   global highchannel

    catch {:modify OVERLAY :delete T}
    :panel OVERLAY :layout L :padx 5 :pady 0 :w $ovlyw :h $ovlyh :before SPECTRA
#    [:path OVERLAY] configure -relief raised -bd 1

    start-ecc
    set highchannel [expr [llength [inform ecc inquire chanlist]] - 1]

    make-overlaypanel
}

proc click-low v {
   global lowchannel highchannel maxchannel
   set lowchannel $v
   if {$lowchannel<0} then {set lowchannel 0}
   if {$lowchannel>$maxchannel} then {set lowchannel $maxchannel}
   if {$highchannel<$lowchannel} then {set highchannel $lowchannel}
   repaint
}

proc click-high v {
   global lowchannel highchannel maxchannel
   set highchannel $v
   if {$highchannel<0} then {set highchannel 0}
   if {$highchannel>$maxchannel} then {set highchannel $maxchannel}
   if {$lowchannel>$highchannel} then {set lowchannel $highchannel}
   repaint
}

proc click-qualifier v {
   global qualifier spectrumlist
   set qualifier $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc click-deselect-all {} {
   global spectrumlist
   set spectrumlist {}
   :modify LIST :select -1
}

proc click-select-all {} {
   global env dirlist spectrumlist
   read-spectra
   paint-histlist
   set i 0
   :modify LIST :show F
   foreach s $dirlist {:modify LIST :select $i; incr i}
   :modify LIST :show T
   set spectrumlist $dirlist
   repaint
}

proc click-adc {v adc} {
# click-adc  select an ADC and deselect the list
   global $adc spectrumlist
   set $adc $v
   if {$spectrumlist != {}} then {paint-histlist; repaint}
}

proc read-qenab {} {
   global sname EGallow EGreport

   set egreport $EGreport; set EGreport -1

   set egallow $EGallow
   set EGallow 0x2000a
   
   for {set v [ersread $sname.qualifierEnable]} {$v=="" || $v==0x2000a} {} {set v [ersread $sname.qualifierEnable]}

   set EGallow $egallow

# This is a hack to synchronise the qualifierEnables in all current systems
   click-qenab $v
   set EGreport $egreport
   return $v
}

proc repaint {} {
  global resource lowchannel highchannel qualifier

     :modify SYSTEM :value $resource
     catch {
       :modify DET-LOW :value $lowchannel
       :modify DET-HIGH :value $highchannel
     }

     :modify ENABLED :value [singles-is-enabled]
     :modify DET-QENAB :value [read-qenab]
     repaint_overlay
}

proc click-redisplay {} {
   read-spectra
   paint-histlist
   repaint
}

proc click arg {
   global debug hsdir
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {click-quit $f; return}
   switch -glob $w {
      ZERO              {click-zero}
      CREATE            {click-create}
      DELETE            {click-delete}
      SHOW              {click-show}
      SAVE              {inform speccopy start singles}
      ADDRESS           {click-addresses}
      SYSTEM -
      SYSTEM-MENU	{click-system $v}
      PATTERN -
      PATTERN-MENU      {click-pattern $v}
      LIST              {click-list $c $v}
      DESELECT-ALL      {click-deselect-all}
      SELECT-ALL        {click-select-all}
      REDISPLAY         {click-redisplay}
      DET-LOW           {click-low $v}
      DET-HIGH          {click-high $v}
      DET-*             {click-overlay $w $v}
      ECC-ENABLE	{click-enable $v}
      ECC-TAC-RANGE-CTRL {click-TAC-range $v}
      ECC-TAC-STOP-CTRL	 {click-TAC-stop $v}
      ECC-TIM-SRC-CTRL	 {click-tim-mode $v}
      default    {puts stdout $arg}
   }
}

proc start args {
   global env started
   global resources
   if {!$started} then {
      make-frame
      catch {make-resource-list}
      :modify SYSTEM-MENU :menu $resources
      catch {maybe-define-singles-registers; click-timing 0}

      paint_detector
      set started 1
      click-redisplay
   } else {
      open-frame
      catch {maybe-define-singles-registers}
   }
}

proc click-quit {f} {
   global env started
   if {$f=="SEL"} then {destroy-frame SEL; return}
   destroy-frame SEL
   close-frame
}

proc finish args {destroy-frame SEL; destroy-frame; set started 0}

proc make-frame {} {
  global env resources patternmenu
  global ovlyw ovlyh

   :frame [self frame-name] :label "[self frame-title]" :show-footer T  :resize F \
        :icon-image "@$env(TCLHOME)/bitmaps/spectrum" \
        :icon-name "[self icon-title]"

   :panel P1 :layout T :padx 10 :pady 10
      :layout H; :col-gap 5

      :hskip 50
      :menubutton SYSTEM-MENU :label "System:" :menu $resources
      :text SYSTEM :label "" :w 12
      :hskip 15
      :choice ENABLED :label "Enabled on ECC cards?" :strings [list "No" "Yes" "Partial"] :deaf T

    :panel SPECTRA :layout L :padx 5 :pady 0
      :list LIST :cols 16 :rows 12 :exclusive F

    :panel P4 :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :row-gap 10
      :col-gap 10
      :button CREATE  :label Create
      :button DELETE  :label Delete
      :button ZERO    :label Zero
      :button SAVE    :label Save...
      :button BROWSER :label "Spectrum Viewer..." :command "inform sb singlesstart"
      :hskip 10
      :button HELP  :label Help

      :next-row T
      :col-gap 10
      :button SHOW          :label Show
      :button SELECT-ALL    :label "Select all"
      :button DESELECT-ALL  :label "Deselect all"

      :menubutton PATTERN-MENU  :label "Selection"  :menu $patternmenu
      :col-gap 0
      :text PATTERN :label "" :w 8

      :hskip 10
      :button REDISPLAY     :label Redisplay

    :panel OVERLAY :layout L :padx 5 :pady 0 :w $ovlyw :h $ovlyh :before SPECTRA
     [:path OVERLAY] configure -relief raised -bd 1

    :show T
}

# *************************************************************
# Now follows the "overlay"
# *************************************************************
#    hist task code overlay for detector type "singles (i.e. from ecc/hrge8x)"


set lowrange 1
set highrange 1
set maxrange 65536
set rangeofitem 16384

set singlesadc 6MeV

set segnames {A B C D}
set seg 15

set nummenu {1 2 4 8 16 32 64 128}
set numhists 1
set rangemenu {128 256 512 1024 2K 4K 8K 16K 32K 64K}
set buffnames {"DMA Buffer" "Link 0" "Link 1" "Link 2" "Link 3"}

set timing 0

set sname Singles

set ProcIndex 0
set CurrentProc singles
set ProcessorList {singles1}
set HostList {singles1}

set total_bits 17
set static_adc_bits 14
set default_adc_bits 14
set adc_bits 13
set saved_adc_bits 0
set tim_bits 4
set saved_tim_bits 4
set hstperseg 1

set modulename ECC

set sharcmem 0

set SourceIndex 0
set LinkIndex [expr $SourceIndex - 1]

set meminc 0

set dumpstart 0
set dumpwords 64

set memOffset 0x0000

set PMC_SCTL		0xc2000000
set SHARC_IOP		0xc2200000
set SHARC_PIO		0xb0001000

set SHARC_SHARC_BASE	0x28400
set LINK_OVERRUN_TOP	0x12
set PCTL_MBOX_BASE	0x40

# base address of link buffer 0
set SHARC_LK_BUFF_BASE	0x20400

set IOP_SYSCON		0x00000000
set ASP_SYSCON		0x0000e000
# for 48-bit access
set ASP_SYSCON_LK_RD	0x0000e028
	
for {set i 0} {$i<8} {incr i} {
   set PCTL_MBOX$i [format "0x%x" [expr $PCTL_MBOX_BASE + (4*$i)]]
}

set PCTL_PCI2SHARC 0x60
set PCTL_SHARC2PCI 0x64

set PPC_SHARC_BASE [format "0x%08x" [expr $SHARC_IOP + ($SHARC_SHARC_BASE << 2)]]
set PPC_LK_BUFF_BASE [format "0x%08x" [expr $SHARC_IOP + ($SHARC_LK_BUFF_BASE << 2)]]

for {set i 0} {$i<4} {incr i} {
#   set LINK_OVERRUN$i [format "0x%02x" [expr ($LINK_OVERRUN_TOP + $i) << 2]]
   set LINK_BUFFER$i [format "0x%02x" [expr ($i * 256) << 2]]
}

set DMA_BUF [format "0x%08x" [expr 64*1024*1024 - 64*1024]]

set SinglesRegisters [list \
   "qualifierEnable generic -n qualifierEnable -t int" \
   "adcLSB generic -n adcLSB -t int" \
   "timingLSB generic -n timingLSB -t int" \
   "adcBitsDef generic -n adcBitsDef -t int" \
   "adcBits generic -n adcBits -t int" \
   "timingBits generic -n timingBits -t int" \
   "DMABuffer bMEM -a$DMA_BUF -o$memOffset -r0" \
   "MailBox0 xMEM -a$SHARC_PIO -o$PCTL_MBOX0 -r1 -ro" \
   "MailBox1 xMEM -a$SHARC_PIO -o$PCTL_MBOX1 -r1 -ro" \
   "MailBox2 xMEM -a$SHARC_PIO -o$PCTL_MBOX2 -r1 -ro" \
   "MailBox3 xMEM -a$SHARC_PIO -o$PCTL_MBOX3 -r1 -ro" \
   "MailBox4 xMEM -a$SHARC_PIO -o$PCTL_MBOX4 -r1 -ro" \
   "MailBox5 xMEM -a$SHARC_PIO -o$PCTL_MBOX5 -r1 -ro" \
   "MailBox6 xMEM -a$SHARC_PIO -o$PCTL_MBOX6 -r1 -ro" \
   "MailBox7 xMEM -a$SHARC_PIO -o$PCTL_MBOX7 -r1 -ro" \
   "PCI2Sharc xMEM -a$SHARC_PIO -o$PCTL_PCI2SHARC -r1" \
   "Sharc2PCI xMEM -a$SHARC_PIO -o$PCTL_SHARC2PCI -r1" \
   "Sharc.IOP_SysCon xMEM -a$SHARC_IOP -o$IOP_SYSCON -m16 -zx -r1" \
   "Sharc.Memory xMEM -a$PPC_SHARC_BASE -o0 -zx -r1" \
   "LookupTable bMEM -a$PPC_SHARC_BASE -o0 -zx -r1 -l 88" \
   "Link.OverRun0 xMEM -a$PPC_SHARC_BASE -o0 -zd -r1" \
   "Link.OverRun1 xMEM -a$PPC_SHARC_BASE -o4 -zd -r1" \
   "Link.OverRun2 xMEM -a$PPC_SHARC_BASE -o8 -zd -r1" \
   "Link.OverRun3 xMEM -a$PPC_SHARC_BASE -o12 -zd -r1" \
   "Link.BadFormat0 xMEM -a$PPC_SHARC_BASE -o16 -zd -r1" \
   "Link.BadFormat1 xMEM -a$PPC_SHARC_BASE -o20 -zd -r1" \
   "Link.BadFormat2 xMEM -a$PPC_SHARC_BASE -o24 -zd -r1" \
   "Link.BadFormat3 xMEM -a$PPC_SHARC_BASE -o28 -zd -r1" \
   "Link.Buffer0 xMEM -a$PPC_LK_BUFF_BASE -o$LINK_BUFFER0 -zd -r1" \
   "Link.Buffer1 xMEM -a$PPC_LK_BUFF_BASE -o$LINK_BUFFER1 -zd -r1" \
   "Link.Buffer2 xMEM -a$PPC_LK_BUFF_BASE -o$LINK_BUFFER2 -zd -r1" \
   "Link.Buffer3 xMEM -a$PPC_LK_BUFF_BASE -o$LINK_BUFFER3 -zd -r1"
]

set table(DET-P2S)      {write-sharc PCI2Sharc}
set table(DET-S2P)      {write-sharc Sharc2PCI}

proc make-overlaypanel {} {
   global segnames seg rangeofitem
      :layout H; :row-gap 8; :col-gap 20; 

      :number DET-LOW  :label "Low Detector:"  :w 5 :valuex 15c
      :next-row T
      :number DET-HIGH :label "High Detector:" :w 5 :valuex 15c
      :col-gap 20


      :next-row T
      :choice DET-SEG  :label "Crystal:"       :valuex 15c :exclusive F :strings $segnames :value $seg

      :next-row T
      :choice-stack DET-RANGE :label "Channels:" :strings {16K} :value 0
      :message DET-QMESS1 :label "Exclude:"
      :col-gap 2
      :choice DET-QENAB :label ""  :exclusive F :strings [list "Veto" "Pileup"] :value [read-qenab]
      :layout H; :next-row T; :row-gap 8; :col-gap 10
      :checkbox DET-TIMING :label "Timing Histograms?" :value 0
      :choice-stack DET-NUMBER :label "No. per Crystal:" :strings {1} :rows 4 :value 0
      :next-row T
      :hskip 5; :button DET-EXPERT :label "Expert & Diagnostics Interface" :command "start-EXPERT"
      :hskip 5; :button DET-ECC :label "ECC Control" :command "start-ECC"

}

proc click-on {w v} {
	global debug table  

	if {$debug} then {puts stderr "click on $w $v"}
	set func [first $table($w)]
	set reg  [second $table($w)]

	$func $reg $v
}

#    this proc handles events from widgets created in the proc make-overlaypanel
#    The widgets DET-LOW; DET-HIGH are handled in the main click procedure
#    and are not required here

proc click-overlay {w v} {
   global memOffset meminc SourceIndex
   switch -glob $w {
      DET-SEG		{click-segment $v}
      DET-RANGE		{click-range $v}
      DET-QENAB		{click-qenab $v}
      DET-TIMING	{click-timing $v}
      DET-NUMBER	{click-hstperseg $v}
      DET-PROCESSOR	{click-processor $v}
      DET-ADC-LSB	{click-lsb "adc" $v}
      DET-TIM-LSB	{click-lsb "timing" $v}
      DET-DEF-ADC-BITS	{click-default-adc-bits $v}
      DET-MEMADDR	{click-memaddr $v;update-sharcmem}
      DET-SOURCE	{click-source $v}
      DET-MEMOFFSET	{click-offset $v}
      DET-INCR		{click-incr $v}
      DET-EMPTY		{:modify DET-MEMDISPLAY :empty T}
      DET-REDISPLAY	{catch {maybe-define-singles-registers};click-redisplay-expert}
      DET-P2S		-
      DET-S2P		{click-on $w $v}
      default	{puts stdout "$w $v"}
   }
}

proc repaint_overlay {} {
   global spectrumlist timing adc_bits
   global seg

   if {[catch {:path DET-RANGE}]} {return}

   if {[llength $spectrumlist] > 0} then {
       :modify DET-SEG      :value 0
   } else {
       :modify DET-SEG      :value $seg
   }
   if {$timing} then {
      :modify DET-RANGE :value [expr $adc_bits - 7]
   }
}

proc get-spectrum-list {} {
        global spectrumlist singlesadc timing hstperseg
        global lowchannel highchannel
        global seg segnames modulename

        if {$spectrumlist != {}} then {return $spectrumlist};  # return names selected from spectrum list

        set seglist [list-take $segnames  [bits-in $seg]]

        set list {}

        for {set i $lowchannel} {$i<=$highchannel} {incr i} {
# append  spectrum names
           set adc $singlesadc
           foreach s $seglist {
              if {$timing} then {
                 for {set t 0} {$t<$hstperseg} {incr t} {
                    lappend list $modulename$i.$s.$adc.t$t
                 }
              } else {
                 lappend list $modulename$i.$s.$adc
              }
           }
        }
        return $list
}

proc click-segment {v} {
   global spectrumlist
   global seg
   set seg $v
   if {$spectrumlist!={}} then {paint-histlist; repaint}
}

proc click-range v {
   global timing CurrentProc resource sname adc_bits default_adc_bits
   global lowrange highrange maxrange rangeofitem tim_bits total_bits hstperseg

   if {$timing} then {
      set adc_bits [expr $v + 7]
   } else {
      set adc_bits $default_adc_bits
   }

   set rangeofitem [expr int(pow(2, $adc_bits))]
   if {$rangeofitem<1} then {set rangeofitem 1}
   if {$rangeofitem>$maxrange} then {set rangeofitem $maxrange}

   if {[expr $tim_bits + $adc_bits] > $total_bits} then {
      midas-warning "You shouldn't have so many channels per spectrum when you have [expr int(pow(2, $tim_bits))] timing spectra"
   }

   set keepproc $CurrentProc
   set CurrentProc $resource
   erswrite $sname.adcBits $adc_bits
   set CurrentProc $keepproc
   repaint
   maybe-update-lsbs-bits
}

proc click-hstperseg {v} {
   global sname tim_bits adc_bits total_bits hstperseg rangeofitem
   global CurrentProc resource
   set tim_bits $v
   set hstperseg [expr int(pow(2, $tim_bits))]
   if {[expr $tim_bits + $adc_bits] > $total_bits} then {
      midas-warning "You shouldn't have so many timing spectra when you have [expr int(pow(2, $adc_bits))] channels per spectrum"
   }

   set keepproc $CurrentProc
   set CurrentProc $resource
   erswrite $sname.timingBits $tim_bits
   set CurrentProc $keepproc
   maybe-update-lsbs-bits
}

proc click-timing {v} {
   global timing tim_bits saved_tim_bits adc_bits saved_adc_bits
   global rangemenu nummenu

   set timing $v
   if {$timing} {
      if {$saved_adc_bits} {set adc_bits $saved_adc_bits}
      set tim_bits $saved_tim_bits
      if {![catch {:path DET-RANGE}]} then {
         :modify DET-RANGE :strings $rangemenu
         :modify DET-NUMBER :strings $nummenu :value $tim_bits
      }
      click-range [expr $adc_bits - 7]
   } else {
      set saved_adc_bits $adc_bits
      set saved_tim_bits $tim_bits
      set tim_bits 0
      if {![catch {:path DET-RANGE}]} then {
         :modify DET-RANGE :strings "16K" :value 0
         :modify DET-NUMBER :strings {1} :value $tim_bits
      }
      click-range 0
   }
   click-hstperseg $tim_bits
   if {![catch {:path DET-RANGE}]} then {
      repaint
   }
   maybe-update-lsbs-bits
}

proc range-of n {
  global rangeofitem timing
# force default ADC bits
  if {!$timing} then {click-range 0}
  return $rangeofitem
}

#
# *************************************************************************
#
# From here on down applies to the EXPERT subframe
#
# *************************************************************************
#

set expertframe [self frame-name].expert
set expertvisible 0

proc click-redisplay-expert {} {
   global expertvisible expertframe
   if {![winfo ismapped .]} then {return}

   make-processor-list
   update-lsbs-bits
   update-sharc
#   update-sharcmem
   update-others
}

proc update-sharcmem {} {
   global sname sharcmem
   erswta $sname.Sharc.Memory -o [expr $sharcmem << 2]
   set v [ersread $sname.Sharc.Memory]
   :modify DET-MEMCONT :value $v
}

proc click-processor {n} {
  global ProcIndex
  set ProcIndex $n

  click-redisplay-expert
}

proc click-memaddr {v} {
   global sharcmem
   set sharcmem $v
}

proc do-memmod {v} {
   global sharcmem
   incr sharcmem $v
   :modify DET-MEMADDR :value [format 0x%08x $sharcmem]
}

proc click-lsb {n v} {
   global sname
   erswrite $sname.[set n]LSB $v
}

proc click-qenab {v} {
   global sname
   global CurrentProc resource
   set keepproc $CurrentProc
   set CurrentProc $resource
   erswrite $sname.qualifierEnable $v
   set CurrentProc $keepproc
}

proc click-default-adc-bits {v} {
   global resource sname default_adc_bits CurrentProc
   set default_adc_bits $v
   set keepproc $CurrentProc
   set CurrentProc $resource
   erswrite $sname.adcBitsDef $default_adc_bits
   set CurrentProc $keepproc
}

proc click-source {n} {
   global SourceIndex LinkIndex dumpwords
   set SourceIndex $n
   set LinkIndex [expr $SourceIndex - 1]
   if {$SourceIndex} then {
      set label "Read Link"
      set dumpwords 128
   } else {
      set label "Read Buffer"
      set dumpwords 64
   }
   :modify DET-RDBUFF :label $label
   update-others
}

proc click-offset {v} {
   set_offset $v
}

proc click-incr {v} {
   global SourceIndex meminc
   set m 0
   if {!$SourceIndex} then {
      set meminc $v
      set m $v
   }
   :modify DET-INCR :value $m
}

proc set_offset {v} {
   global memOffset SourceIndex
   set m 0x0000
   if {!$SourceIndex} then {
      set memOffset [format 0x%04x $v]
      set m $memOffset
   }
   :modify DET-MEMOFFSET :value $m
}

proc zero_offset {} {set_offset 0}

proc do_readLUT {} {
   global sname
   global EGreport

   set egreport $EGreport; set EGreport -1

   fixup_LookupTable_address
   set register $sname.LookupTable
   set data [string trimleft [ersread $register]]

   set addr  0
   for {set i 0} {$i<88} {incr i} {
      if {![expr $i % 8]} {
         :modify DET-MEMDISPLAY :strings "\n"
         :modify DET-MEMDISPLAY :strings [format "0x%04x: " $addr]; incr addr 32
      }
      :modify DET-MEMDISPLAY :strings [format "0x%08x " [lindex $data $i]]
   }
   :modify DET-MEMDISPLAY :strings "\n"

   set EGreport $egreport
}

proc do_readRAM {} {
   global sname
   global memOffset meminc
   global dumpwords EGreport

   set egreport $EGreport; set EGreport -1

   set register $sname.DMABuffer

   erswta $register "-o $memOffset -l $dumpwords"
   set data [string trimleft [ersread $register]]

   :modify DET-MEMDISPLAY :strings "\n     Memory Offset $memOffset"
   set addr  $memOffset

   for {set i 0} {$i<$dumpwords} {incr i} {
      if {![expr $i % 8]} {
         :modify DET-MEMDISPLAY :strings "\n"
         :modify DET-MEMDISPLAY :strings [format "0x%04x: " $addr]; incr addr 32
      }
      :modify DET-MEMDISPLAY :strings [format "0x%08x " [lindex $data $i]]
   }
   :modify DET-MEMDISPLAY :strings "\n"

   if {$meminc != 0} then {
      set_offset [expr $memOffset + 256]
   }

   set EGreport $egreport
}

proc do_readLink {l} {
   global sname ASP_SYSCON_LK_RD ASP_SYSCON LINK_BUFFER$l
   global dumpwords EGreport

   set egreport $EGreport; set EGreport -1

   set LINK_BUFFER [set LINK_BUFFER$l]

   erswta $sname.Sharc.IOP_SysCon -m16
   erswrite $sname.Sharc.IOP_SysCon $ASP_SYSCON_LK_RD
   erswta $sname.Link.Buffer$l -o $LINK_BUFFER
   ersread $sname.Link.Buffer$l
   ersread $sname.Link.Buffer$l

   :modify DET-MEMDISPLAY :strings "\n     Link Buffer $l"
   set addr 0

   for {set i 0} {$i<$dumpwords} {incr i} {
      if {![expr $i % 4]} {
         :modify DET-MEMDISPLAY :strings "\n"
         :modify DET-MEMDISPLAY :strings [format "%3d: " $addr]; incr addr 4
      }
      set lo [format %04x [expr [ersread $sname.Link.Buffer$l]&0xffff]]
      set mi [format %04x [expr [ersread $sname.Link.Buffer$l]&0xffff]]
      set hi [format %04x [expr [ersread $sname.Link.Buffer$l]&0xffff]]
      set res [join [list 0x $hi $mi $lo] ""]
      :modify DET-MEMDISPLAY :strings "$res "
      erswta $sname.Link.Buffer$l -o [incr LINK_BUFFER 4]
   }
   erswrite $sname.Sharc.IOP_SysCon $ASP_SYSCON
   erswta $sname.Sharc.IOP_SysCon -m0
   erswrite $sname.Sharc.IOP_SysCon $ASP_SYSCON
   :modify DET-MEMDISPLAY :strings "\n"

   set EGreport $egreport
}

proc click-readBuff {} {
   global SourceIndex LinkIndex
   if {$SourceIndex} then {do_readLink $LinkIndex} else {do_readRAM}
   click-redisplay-expert
}

proc click-readcurr {} {
   global SourceIndex LinkIndex
   if {$SourceIndex} then {
      do_readLink $LinkIndex
   } else {
      global sname dumpwords
      set offset [ersread $sname.MailBox0]
      set_offset $offset
      :modify DET-MBOX0  :value $offset
      do_readRAM
   }
   click-redisplay-expert
}

proc write-sharc {r v} {
   global debug sname

   if {$debug} then {puts stderr "write-sharc $sname.$r $v"}
   erswrite $sname.$r $v
}

proc start-EXPERT {} {
   global expertvisible expertframe
   if {$expertvisible==0} then {make-expert} else {open-frame $expertframe}
   catch {maybe-define-singles-registers}
   click-redisplay-expert   
}

proc make-expert {} {
   global env expertframe expertvisible
   global ProcessorList buffnames SourceIndex memOffset default_adc_bits
   if {$SourceIndex} then {
      set label "Read Link"
   } else {
      set label "Read Buffer"
   }

  :frame "$expertframe" :label "Singles Setup & Diagnostics (expert)" :resize T :transient T

# N.B.
# MailBox 0 PCI DMA buffer number
# MailBox 1 Number of buffers sent
# MailBox 2 Address of LUT
# MailBox 3 Date & Version
# MailBox 4 PCI DMA status
# MailBox 5 Unallocated
# MailBox 6 Unallocated
# MailBox 7 Unallocated
#      :text DET-MBOX0  :label "PCI DMA buffer number:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX4  :label "PCI DMA status:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX1  :label "Number of buffers sent:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX5  :label "MailBox 5:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX2  :label "Address of LUT:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX6  :label "MailBox 6:" :deaf T :valuex 115 :w 10
#      :next-row T
#      :text DET-MBOX3  :label "Date & Version:" :deaf T :valuex 170 :w 10
#      :text DET-MBOX7  :label "MailBox 7:" :deaf T :valuex 115 :w 10


   :panel P20  :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 10
      :choice-stack DET-PROCESSOR :label "Processor Name:" :strings $ProcessorList
      :hskip 32
      :text DET-HOST :label "Host Name" :w 8 :deaf T
      :next-row T
      :number DET-ADC-LSB  :label "ADC LSB:" :min 0 :w 5
      :number DET-TIM-LSB  :label "Timing LSB:" :min 0 :w 5
      :text DET-ADC-BITS  :label "ADC bits:" :w 5 :deaf T
      :text DET-TIM-BITS  :label "Timing bits:" :w 5 :deaf T
      :text DET-DEF-ADC-BITS  :label "Default ADC bits:" :w 5 :value $default_adc_bits

   :panel MBOXES  :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 15
      :text DET-MBOX0  :label "PCI DMA buffer number:" :deaf T :valuex 150 :w 10
      :text DET-MBOX1  :label "Buffers sent:" :deaf T :valuex 95 :w 10
      :text DET-MBOX2  :label "LUT Address:" :deaf T :valuex 95 :w 10
      :text DET-MBOX3  :label "Date/Version:" :deaf T :valuex 95 :w 10
      :next-row T
      :text DET-MBOX4  :label "PCI DMA status:" :deaf T :valuex 150 :w 10
      :text DET-MBOX5  :label "Statistics Addr:" :deaf T :valuex 95 :w 10
      :text DET-MBOX6  :label "MailBox 6:" :deaf T :valuex 95 :w 10
      :text DET-MBOX7  :label "MailBox 7:" :deaf T :valuex 95 :w 10

   :panel DOORBELLS  :layout T :padx 10 :pady 10
      :layout H; :row-gap 0; :col-gap 10
      :message DET-DOORBELLS :label "Doorbells:" :bold T
      :text DET-P2S  :label "PCI to Sharc:" :valuex 150 :w 10
      :col-gap 20
      :text DET-S2P  :label "Sharc to PCI:" :valuex 84 :w 10

   :panel LINKS  :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :message DET-LINKS0 :label "Link Statistics" :bold T
      :next-row T
      :text DET-OVERRUN0 :label "Overruns  0:" :w 10 :valuex 100 :deaf T
      :text DET-OVERRUN1 :label "1:" :w 10 :deaf T
      :text DET-OVERRUN2 :label "2:" :w 10 :deaf T
      :text DET-OVERRUN3 :label "3:" :w 10 :deaf T
      :next-row T
      :text DET-BADFORMAT0 :label "Bad Format  0:" :w 10 :valuex 100 :deaf T
      :text DET-BADFORMAT1 :label "1:" :w 10 :deaf T
      :text DET-BADFORMAT2 :label "2:" :w 10 :deaf T
      :text DET-BADFORMAT3 :label "3:" :w 10 :deaf T

#   :panel MEMORY  :layout T :padx 10 :pady 10
#      :layout H; :row-gap 0; :col-gap 10
#      :message DET-MES1  :label "Inspect SHARC memory" :bold T
#      :next-row T
#      :text DET-MEMADDR  :label " Address:" :value 0x000000 :w 10
#      :text DET-MEMCONT  :label "Contents:" :deaf T :w 10
#      :button DET-READ :label "Read" :command "update-sharcmem"
#      :col-gap 2
#      :message DET-MES2  :label "Change Address:"
#      :button DET-MEMDEC  :label "Dec" :command "do-memmod -1;update-sharcmem"
#      :button DET-MEMINC  :label "Inc" :command "do-memmod 1;update-sharcmem"

   :panel DISPLAY :layout B :padx 10 :after FOOTER
      :layout H; :row-gap 10; :col-gap 15
      :choice-stack DET-SOURCE :label "Read From:" :strings $buffnames :value $SourceIndex
      :text DET-MEMOFFSET :label "Offset:" :w 6 :value "$memOffset" :x 200
      :button DET-ZERO :label "Zero Offset" :command "zero_offset"
      :checkbox DET-INCR :label "Increment Offset" :value 0
      :next-row T
      :galley DET-MEMDISPLAY :rows 11 :cols 96 :font fixed

   :panel CLICK :layout B :padx 10 :pady 10 :after FOOTER
      :layout H; :row-gap 15; :col-gap 15
      :button DET-REDISPLAY :label "Redisplay"
      :button DET-RDBUFF :label $label :command "click-readBuff"
      :button DET-RDCURR :label "Read Current" :command "click-readcurr"
      :button DET-EMPTY     :label "Empty memory display"
      :hskip 100
      :button DET-DEFINE     :label "Define Registers" :command "force-define-singles-registers"

   :panel P21 :layout T :padx 5 :pady 5

  :show T
 set expertvisible 1
}

# due to a bug (in CES LynxOS?) the read sometimes returns nothing
# and sometimes "application detected failure" (0x2000a/131082)
proc update-lsbs-bits {} {
   global sname EGallow
   global default_adc_bits static_adc_bits

   set egallow $EGallow
   set EGallow 0x2000a
   
   for {set v [ersread $sname.adcLSB]} {$v=="" || $v==0x2000a} {} {set v [ersread $sname.adcLSB]}
   :modify DET-ADC-LSB  :value $v

   for {set v [ersread $sname.timingLSB]} {$v=="" || $v==0x2000a} {} {set v [ersread $sname.timingLSB]}
   :modify DET-TIM-LSB  :value $v

   for {set v [ersread $sname.adcBits]} {$v=="" || $v==0x2000a} {} {set v [ersread $sname.adcBits]}
   :modify DET-ADC-BITS  :value $v

   for {set v [ersread $sname.timingBits]} {$v=="" || $v==0x2000a} {} {set v [ersread $sname.timingBits]}
   :modify DET-TIM-BITS  :value $v

   for {set v [ersread $sname.adcBitsDef]} {$v=="" || $v==0x2000a} {} {set v [ersread $sname.adcBitsDef]}
   if {$v==0} {set default_adc_bits $static_adc_bits} {set default_adc_bits $v}
   :modify DET-DEF-ADC-BITS  :value $default_adc_bits

   set EGallow $egallow
}

proc maybe-update-lsbs-bits {} {
   if {![catch {:path DET-ADC-LSB}]} {update-lsbs-bits}
}

proc get-buffers-sent {args} {
   global sname CurrentProc EGallow
   
   if {[llength $args]} {set p [first $args]} {set p singles}

   set keepproc $CurrentProc
   set CurrentProc $p
   set ega $EGallow; set EGallow 0x0
   set z [catch {ersread $sname.MailBox1} m]
   if {$z != 0} then {insert-log "Error $m reading $sname.MailBox1"; set m 0}
   set EGallow $ega
   set CurrentProc $keepproc
   return $m
}

# This procedure writes 1 to the Sharc doorbell - use with care
proc initialise-links {args} {
   global sname CurrentProc EGreport EGallow

   catch {maybe-define-singles-registers}
   if {[llength $args]} {set p [first $args]} {set p singles}

   set keepproc $CurrentProc
   set CurrentProc $p

   set egallow $EGallow; set EGallow 0x10002
   set egreport $EGreport; set EGreport -1
   set z [catch {write-sharc PCI2Sharc 1} m]
   if {$z != 0} then {
      midas-error $m
   }
   set EGreport $egreport
   set EGallow $egallow
   set CurrentProc $keepproc
   return
}

proc update-sharc {} {
   global sname EGallow

   for {set i 0} {$i<8} {incr i} {
      set v [ersread $sname.MailBox$i]
      :modify DET-MBOX$i  :value $v
   }
   set v [ersread $sname.PCI2Sharc]
   :modify DET-P2S  :value $v
   set v [ersread $sname.Sharc2PCI]
   :modify DET-S2P  :value $v
}

proc update-others {} {
   global memOffset meminc sname LinkIndex

   if {$LinkIndex >= 0} then {
      set m 0x0000
      set i 0
      :modify DET-MEMOFFSET  :deaf T
   } else {
      set m $memOffset
      set i $meminc
   }

   :modify DET-MEMOFFSET  :value $m
   :modify DET-INCR  :value $i

   fixup_statistics_addresses
   for {set i 0} {$i<4} {incr i} {
      :modify DET-OVERRUN$i  :value [ersread $sname.Link.OverRun$i]
      :modify DET-BADFORMAT$i  :value [ersread $sname.Link.BadFormat$i]
   }
}

proc make-processor-list {} {
  global expertvisible
  global ProcessorList HostList resource
  global CurrentProc CurrentHost ProcIndex

#    find processors available

   if {[catch {eg "(experiment ?resources)"} m]} then {
      midas-warning "no experiment connected!!!"
      set ProcessorList {singles1}
      set HostList {singles1}
      set ProcIndex 0
   } else {
      set ProcessorList [make-expert-resource-list "name" $m $resource]
      set HostList  [make-expert-resource-list "host" $m $resource]
      if {$ProcIndex >= [llength $ProcessorList]} then {set ProcIndex 0}
   }
#    sort lists into sensible order
   set proclist [lsort $ProcessorList]
   set hostlist {}
   foreach c $proclist {
      lappend hostlist [lindex $HostList [lsearch $ProcessorList $c]]
   }
   set ProcessorList $proclist
   set HostList $hostlist
   set CurrentProc     [lindex $ProcessorList $ProcIndex]
   set CurrentHost     [lindex $HostList $ProcIndex]

   if {$expertvisible} then {
      :modify DET-PROCESSOR :strings $ProcessorList :value $ProcIndex
         :modify DET-HOST  :value $CurrentHost
   }
}

proc make-expert-resource-list {class rlist resource} {
   set ll {}
   foreach item $rlist {
      set l [find-resource $class $item $resource]
      if {$l!={}} then {lappend ll $l}
   }
   return $ll
}

proc find-resource {class ps resource} {
   set l {}
   set role {}; set name {}; set host {}
   foreach p $ps {
      switch [first $p] {
         name {set name [second $p]}
         role {set role [second $p]}
         host {set host [second $p]}
         type {set type [second $p]}
         default {}
      }
   }
   if {$role==$resource && $type=="register"}  then  {set l [set $class]}
   return $l
}

proc ersdefine {register class} {
   global CurrentProc
   EG "register :name '/$CurrentProc/$register' :class '$class' !new"
}

proc ersinquire {register} {
   global CurrentProc
   eg "(register :pattern '/$CurrentProc/$register' ?names)"
}

proc erswta {register args} {
   global CurrentProc
   EG "register  :name '/$CurrentProc/$register' :object '$args'"
}

proc ersreads {registers} {
   global CurrentProc EGreport

   set egreport $EGreport; set EGreport -1
   set nv [string trimright [EG "register  :pattern '/$CurrentProc/$registers' ?names-and-values"]]
   set EGreport $egreport
   return $nv
}

proc ersread {register} {
   global CurrentProc EGreport

   set egreport $EGreport; set EGreport -1
   set v [string trimright [EG "register  :name '/$CurrentProc/$register' ?value"]]
   set EGreport $egreport
   return $v
}

# Beware this uses :pattern - so it could write to ALL hosts with 1 call
proc erswrite {register val} {
   global debug CurrentProc
   if {$debug} then {puts stderr "erswrite: EG register  :pattern '/$CurrentProc/$register' :value '$val'"}
   EG "register  :pattern '/$CurrentProc/$register' :value '$val'"
}


# The base address of the "statistics registers" is found in mbox5
proc fixup_statistics_addresses {} {
   global sname SHARC_SHARC_BASE
   set stats [ersread $sname.MailBox5]
   for {set i 0} {$i<4} {incr i} {       
      set off [format "0x%04x" [expr ($stats   + $i - $SHARC_SHARC_BASE) << 2]]
      erswta $sname.Link.OverRun$i "-o $off"

      set off [format "0x%04x" [expr ($stats+4 + $i - $SHARC_SHARC_BASE) << 2]]
      erswta $sname.Link.BadFormat$i "-o $off"
   }
}

# The base address of the Lookup Table is found in mbox2
proc fixup_LookupTable_address {} {
   global sname SHARC_SHARC_BASE
   set off [ersread $sname.MailBox2]
   set off [format "0x%04x" [expr ($off - $SHARC_SHARC_BASE) << 2]]
   erswta $sname.LookupTable "-o $off"
}

proc fixup_addresses {} {
   fixup_statistics_addresses
   fixup_BadFormat_addresses
}

proc xfixup_addresses {} {
   global sname SHARC_SHARC_BASE
   set stats [ersread $sname.MailBox5]
   for {set i 0} {$i<4} {incr i} {       
      set off [format "0x%04x" [expr ($stats   + $i - $SHARC_SHARC_BASE) << 2]]
      erswta $sname.Link.OverRun$i "-o $off"

      set off [format "0x%04x" [expr ($stats+4 + $i - $SHARC_SHARC_BASE) << 2]]
      erswta $sname.Link.BadFormat$i "-o $off"
   }

   set off [ersread $sname.MailBox2]
   set off [format "0x%04x" [expr ($off - $SHARC_SHARC_BASE) << 2]]
   erswta $sname.LookupTable "-o $off"
}

proc define-register {name class attributes} {
   global sname  EGreport
   global CurrentProc

   set register $sname.$name

   set z [catch {ersinquire $register} r]
   if {$z != 0} then {return}
   if {$r==""} then {
      if {$EGreport>=0} then {set-footer "defining register $register"}
      if {[catch {ersdefine $register $class} m]} then {
         midas-error $m
      }
   }
   set z [catch {ersinquire $register} r]
   if {$r == $register} then {erswta $register "$attributes"}
   clear-footer
}

proc define-singles-registers {p} {
   global CurrentProc SinglesRegisters
   set keepproc $CurrentProc
   set CurrentProc $p
   foreach r $SinglesRegisters {
      define-register [first $r] [second $r] [lreplace $r 0 1]
   }
   fixup_addresses
   set CurrentProc $keepproc
}

proc maybe-define-singles-registers {} {
   global ProcessorList CurrentProc sname SinglesRegisters EGreport
   set egreport $EGreport; set EGreport -1
   set keepproc $CurrentProc

   foreach p $ProcessorList {
      set CurrentProc $p
      set r [ersinquire $sname.[first [first $SinglesRegisters]]]
      if {$r==""} then {define-singles-registers $p}
   }

   set EGreport $egreport
   set CurrentProc $keepproc
}

proc force-define-singles-registers {} {
   global ProcessorList CurrentProc sname SinglesRegisters EGreport
   set egreport $EGreport; set EGreport -1
   set keepproc $CurrentProc

   foreach p $ProcessorList {
      define-singles-registers $p
   }

   set EGreport $egreport
   set CurrentProc $keepproc
}

# define the registers on the fly when the script is read in
#catch {make-resource-list; maybe-define-singles-registers; click-timing 0}

#
# *************************************************************************
#
# From here on down applies to the ECC subframe
#
# *************************************************************************
#

set eccframe [self frame-name].ecc
set eccvisible 0

set cardcontrol "#\[?,??,???].SinglesReadoutEnable"
set parilcontrol "\[?,??,???].?.PARIL.RCS.SingleEnable"
set timsrc "\[?,??,???].?.PARIL.RCS.TAC_TFG"
set tacstopmode "#\[?,??,???].TACStopMode"
set tacrange "\[?,??,???].?.TAC_Range"


proc click-enable {v} {start-ecc; inform ecc enable-singles $v; repaint_ecc}
proc click-tim-mode {v} {start-ecc; inform ecc set-tim-src $v; repaint_ecc}
proc click-TAC-stop {v} {start-ecc; inform ecc set-TAC-stop $v; repaint_ecc}
proc click-TAC-range {v} {start-ecc; inform ecc set-TAC-range $v; repaint_ecc}

proc click-redisplay-ecc {} {
   repaint_ecc
}

proc start-ECC {} {
   global eccvisible eccframe
   if {$eccvisible==0} then {make-ecc} else {open-frame $eccframe}
   click-redisplay-ecc   
}

proc make-ecc {} {
   global env eccframe eccvisible
   global ProcessorList buffnames SourceIndex memOffset default_adc_bits

  :frame "$eccframe" :label "Singles Global ECC Control" :resize F :transient T

   :panel P30  :layout T :padx 10 :pady 10
      :layout H; :row-gap 4; :col-gap 8

      :message ECC-MESS1 :label "Singles Readout from ECC cards" :bold T
      :next-row T
      :choice ECC-ENABLE :label "Required:" :strings [list "No" "Yes"] :value -1
      :choice ECC-ENABLED :label "Enabled:" :strings [list "None" "All" "Some"] :deaf T
      :next-row T

      :message ECC-MESS2 :label "Time Reference" :bold T
      :next-row T
      :choice ECC-TIM-SRC-CTRL :label "Required:" :strings [list "TAC" "TFG"] :value -1
      :choice ECC-TIM-SRC-STAT :label "Current:" :strings [list "TAC" "TFG"] :exclusive F :deaf T
      :row-gap 10; :next-row T

      :message ECC-MESS3 :label "TAC Stop Mode" :bold T
      :next-row T
      :choice ECC-TAC-STOP-CTRL :label "Required:" :strings [list "FT" "Beam"] :value -1
      :choice ECC-TAC-STOP-STAT :label "Current:" :strings [list "FT" "Beam"] :exclusive F :deaf T
      :next-row T

      :message ECC-MESS4 :label "TAC Range" :bold T
      :next-row T
      :choice ECC-TAC-RANGE-CTRL :label "Required:" :strings [list "200ns" "2us"] :value -1
      :choice ECC-TAC-RANGE-STAT :label "Current:" :strings [list "200ns" "2us"] :exclusive F :deaf T
      :next-row T

      :button ECC-REDISPLAY :label "Redisplay" :command "click-redisplay-ecc"

   :panel P31 :layout T :padx 5 :pady 5

  :show T
 set eccvisible 1

}

proc ersread-ecc-regs {registers} {
   global CurrentProc; set cp $CurrentProc; set CurrentProc VXI
   set v [ersreads $registers]
   set CurrentProc $cp
   return $v
}

proc singles-is-enabled {} {
   global modulename cardcontrol parilcontrol

   set values {}

   set cr $modulename$cardcontrol
   set cnv [ersread-ecc-regs $cr]
   foreach nv $cnv { lappend values [second $nv] }

   set pr $modulename$parilcontrol
   set pnv [ersread-ecc-regs $pr]
   foreach nv $pnv { lappend values [second $nv] }

   set disabled [expr [lsearch $values 0] != -1]
   set enabled [expr [lsearch $values 1] != -1]

# If some are enabled and some are disabled it's partial
# otherwise they are all the same and match the first value
   if {$enabled && $disabled} then {
      return 2
   } else {
      return [first $values]
   }
}

proc ecc-mode {r} {
   global modulename

   set values {}

   set tr $modulename$r
   set tnv [ersread-ecc-regs $tr]
   foreach nv $tnv { lappend values [second $nv] }

   set b0 [expr [lsearch $values 0] != -1]
   set b1 [expr 2 * ([lsearch $values 1] != -1)]

# If some are one value and some are the other it's partial
# otherwise they are all the same and match the first value
   return [expr $b0 | $b1]
}

proc tac-range {} {global tacrange; return [ecc-mode $tacrange]}
proc tac-stop-mode {} {global tacstopmode; return [ecc-mode $tacstopmode]}
proc tim-src {} {global timsrc; return [ecc-mode $timsrc]}

proc repaint_ecc {} {
   :modify ECC-ENABLED :value [singles-is-enabled]
   :modify ECC-TAC-RANGE-STAT :value [tac-range]
   :modify ECC-TAC-STOP-STAT :value [tac-stop-mode]
   :modify ECC-TIM-SRC-STAT :value [tim-src]
}
